/*****************************************************************************
 * HClassBox.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HClassBox
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.4 $
 * $Date: 2003/10/02 09:52:44 $
 *
 */

#ifndef H_CLASS_BOX_H
#define H_CLASS_BOX_H

class LIntExport HClassBox: public HRootObject {
public:
  HClassBox(void);

  ~HClassBox(void);
private:
  // Copying and Assignment not allowed
  HClassBox(const HClassBox &inst);
  HClassBox &operator= (const HClassBox &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HClassBox";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  // Handle
  long  id;
public:
  // Save the classificator in a file.
  virtual void WriteClassBox(const HTuple &FileName) const;
  virtual void WriteClassBox(const char *FileName) const;
  // Set system parameters for classification.
  virtual void SetClassBoxParam(const HTuple &Flag, const HTuple &Value) const;
  virtual void SetClassBoxParam(const char *Flag, double Value) const;
  // Read the classificator from a file.
  virtual void ReadClassBox(const HTuple &FileName) const;
  virtual void ReadClassBox(const char *FileName) const;
  // Train the classificator with one data set.
  virtual void LearnSampsetBox(const HTuple &SampKey, const HTuple &Outfile, const HTuple &NSamples, const HTuple &StopError, const HTuple &ErrorN) const;
  virtual void LearnSampsetBox(long SampKey, const char *Outfile, long NSamples, double StopError, long ErrorN) const;
  // Train the classificator.
  virtual void LearnClassBox(const HTuple &Features, const HTuple &Class) const;
  // Get information about the current parameter.
  virtual double GetClassBoxParam(const HTuple &Flag) const;
  virtual double GetClassBoxParam(const char *Flag) const;
  // Create a new classificator.
  virtual void CreateClassBox(void);
  // Description of the classificator.
  virtual void DescriptClassBox(const HTuple &Dimensions) const;
  virtual void DescriptClassBox(long Dimensions) const;
  // Classify a set of arrays.
  virtual double TestSampsetBox(const HTuple &SampKey) const;
  virtual double TestSampsetBox(long SampKey) const;
  // Classify a tuple of attributes with rejection class.
  virtual long EnquireRejectClassBox(const HTuple &FeatureList) const;
  // Classify a tuple of attributes.
  virtual long EnquireClassBox(const HTuple &FeatureList) const;
  // Train the current classificator using a multi-channel image.
  virtual void LearnNdimBox(const HRegionArray &Foreground, const HRegionArray &Background, const HImageArray &MultiChannelImage) const;
  // Classify pixels using hyper-cuboids.
  virtual HRegionArray ClassNdimBox(const HImageArray &MultiChannelImage) const;
};

#endif
