/*****************************************************************************
 * HComponentModel.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HComponentModel
 *
 * (c) 1996-2003 by MVTec Software GmbH
 *                  www.mvtec.com
 * 
 *****************************************************************************
 *
 * $Revision: 1.1 $
 * $Date: 2003/10/07 13:47:05 $
 *
 */

#ifndef H_COMPONENT_MODEL_H
#define H_COMPONENT_MODEL_H

class LIntExport HComponentModel: public HRootObject {
public:
  HComponentModel(void) {id = -1;}
  HComponentModel(
          const HImage &ModelImage, 
          const HRegionArray &ComponentRegions, 
          const HTuple &VariationRow,
          const HTuple &VariationColumn,
          const HTuple &VariationAngle,
          const HTuple &AngleStart,
          const HTuple &AngleExtent, 
          const HTuple &ContrastLowComp,
          const HTuple &ContrastHighComp,
          const HTuple &MinSizeComp,
          const HTuple &MinContrastComp,
          const HTuple &MinScoreComp,
          const HTuple &NumLevelsComp,
          const HTuple &AngleStepComp,
          const HTuple &OptimizationComp,
          const HTuple &MetricComp,
          const HTuple &PregenerationComp,
          HTuple *RootRanking);
  HComponentModel(
          const HImage &ModelImage, 
          const HRegionArray &ComponentRegions, 
          const long VariationRow,
          const long VariationColumn,
          const double VariationAngle,
          const double AngleStart,
          const double AngleExtent, 
          const long ContrastLowComp,
          const long ContrastHighComp,
          const long MinSizeComp,
          const long MinContrastComp,
          const double MinScoreComp,
          const long NumLevelsComp,
          const double AngleStepComp,
          const char *OptimizationComp,
          const char *MetricComp,
          const char *PregenerationComp,
          HTuple *RootRanking);
  HComponentModel(
          const HComponentTraining &ComponentTrainingID,
          const HTuple &AngleStart,
          const HTuple &AngleExtent,
          const HTuple &MinContrastComp,
          const HTuple &MinScoreComp,
          const HTuple &NumLevelsComp,
          const HTuple &AngleStepComp,
          const HTuple &OptimizationComp,
          const HTuple &MetricComp,
          const HTuple &PregenerationComp,
          HTuple *RootRanking);
  HComponentModel(
          const HComponentTraining &ComponentTrainingID,
          const double AngleStart,
          const double AngleExtent,
          const long MinContrastComp,
          const double MinScoreComp,
          const long NumLevelsComp,
          const double AngleStepComp,
          const char *OptimizationComp,
          const char *MetricComp,
          const char *PregenerationComp,
          HTuple *RootRanking);

  ~HComponentModel(void);
private:
  // Copying and Assignment not allowed
  HComponentModel(const HComponentModel &inst);
  HComponentModel &operator= (const HComponentModel &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HComponentModel";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  //Handle
  long  id;
public:
  // Return the components of a found instance of a component model.
  virtual HRegionArray GetFoundComponentModel(const HTuple &ModelStart, const HTuple &ModelEnd, const HTuple &RowComp, const HTuple &ColumnComp, const HTuple &AngleComp, const HTuple &ScoreComp, const HTuple &ModelComp, const HTuple &ModelMatch, const HTuple &MarkOrientation, HTuple *RowCompInst, HTuple *ColumnCompInst, HTuple *AngleCompInst, HTuple *ScoreCompInst) const;
  virtual HRegionArray GetFoundComponentModel(const HTuple &ModelStart, const HTuple &ModelEnd, const HTuple &RowComp, const HTuple &ColumnComp, const HTuple &AngleComp, const HTuple &ScoreComp, const HTuple &ModelComp, const HTuple &ModelMatch, const HTuple &MarkOrientation, double *RowCompInst, double *ColumnCompInst, double *AngleCompInst, double *ScoreCompInst) const;
  virtual HRegionArray GetFoundComponentModel(long ModelStart, long ModelEnd, double RowComp, double ColumnComp, double AngleComp, double ScoreComp, long ModelComp, long ModelMatch, const char *MarkOrientation, HTuple *RowCompInst, HTuple *ColumnCompInst, HTuple *AngleCompInst, HTuple *ScoreCompInst) const;
  virtual HRegionArray GetFoundComponentModel(long ModelStart, long ModelEnd, double RowComp, double ColumnComp, double AngleComp, double ScoreComp, long ModelComp, long ModelMatch, const char *MarkOrientation, double *RowCompInst, double *ColumnCompInst, double *AngleCompInst, double *ScoreCompInst) const;
  // Find the best matches of a component model in an image.
  virtual HTuple FindComponentModel(const HImage &Image, const HTuple &RootComponent, const HTuple &AngleStartRoot, const HTuple &AngleExtentRoot, const HTuple &MinScore, const HTuple &NumMatches, const HTuple &MaxOverlap, const HTuple &IfRootNotFound, const HTuple &IfComponentNotFound, const HTuple &PosePrediction, const HTuple &MinScoreComp, const HTuple &SubPixelComp, const HTuple &NumLevelsComp, const HTuple &GreedinessComp, HTuple *ModelEnd, HTuple *Score, HTuple *RowComp, HTuple *ColumnComp, HTuple *AngleComp, HTuple *ScoreComp, HTuple *ModelComp) const;
  virtual long FindComponentModel(const HImage &Image, const HTuple &RootComponent, const HTuple &AngleStartRoot, const HTuple &AngleExtentRoot, const HTuple &MinScore, const HTuple &NumMatches, const HTuple &MaxOverlap, const HTuple &IfRootNotFound, const HTuple &IfComponentNotFound, const HTuple &PosePrediction, const HTuple &MinScoreComp, const HTuple &SubPixelComp, const HTuple &NumLevelsComp, const HTuple &GreedinessComp, long *ModelEnd, double *Score, double *RowComp, double *ColumnComp, double *AngleComp, double *ScoreComp, long *ModelComp) const;
  virtual HTuple FindComponentModel(const HImage &Image, long RootComponent, double AngleStartRoot, double AngleExtentRoot, double MinScore, long NumMatches, double MaxOverlap, const char *IfRootNotFound, const char *IfComponentNotFound, const char *PosePrediction, double MinScoreComp, const char *SubPixelComp, long NumLevelsComp, double GreedinessComp, HTuple *ModelEnd, HTuple *Score, HTuple *RowComp, HTuple *ColumnComp, HTuple *AngleComp, HTuple *ScoreComp, HTuple *ModelComp) const;
  virtual long FindComponentModel(const HImage &Image, long RootComponent, double AngleStartRoot, double AngleExtentRoot, double MinScore, long NumMatches, double MaxOverlap, const char *IfRootNotFound, const char *IfComponentNotFound, const char *PosePrediction, double MinScoreComp, const char *SubPixelComp, long NumLevelsComp, double GreedinessComp, long *ModelEnd, double *Score, double *RowComp, double *ColumnComp, double *AngleComp, double *ScoreComp, long *ModelComp) const;
  // Return the search tree of a component model.
  virtual HRegion GetComponentModelTree(HRegionArray *Relations, const HTuple &RootComponent, const HTuple &Image, HTuple *StartNode, HTuple *EndNode, HTuple *Row, HTuple *Column, HTuple *Phi, HTuple *Length1, HTuple *Length2, HTuple *AngleStart, HTuple *AngleExtent) const;
  virtual HRegion GetComponentModelTree(HRegionArray *Relations, const HTuple &RootComponent, const HTuple &Image, long *StartNode, long *EndNode, double *Row, double *Column, double *Phi, double *Length1, double *Length2, double *AngleStart, double *AngleExtent) const;
  // Return the parameters of a component model.
  virtual HTuple GetComponentModelParams(HTuple *RootRanking, HTuple *ShapeModelIDs) const;
  virtual double GetComponentModelParams(long *RootRanking, long *ShapeModelIDs) const;
  // Read a component model from a file.
  virtual void ReadComponentModel(const HTuple &FileName);
  virtual void ReadComponentModel(const char *FileName);
  // Write a component model to a file.
  virtual void WriteComponentModel(const HTuple &FileName) const;
  virtual void WriteComponentModel(const char *FileName) const;
  // Prepare a component model for matching based on explicitly specified
  // components and relations.
  virtual HTuple CreateComponentModel(const HImage &ModelImage, const HRegionArray &ComponentRegions, const HTuple &VariationRow, const HTuple &VariationColumn, const HTuple &VariationAngle, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &ContrastLowComp, const HTuple &ContrastHighComp, const HTuple &MinSizeComp, const HTuple &MinContrastComp, const HTuple &MinScoreComp, const HTuple &NumLevelsComp, const HTuple &AngleStepComp, const HTuple &OptimizationComp, const HTuple &MetricComp, const HTuple &PregenerationComp);
  virtual HTuple CreateComponentModel(const HImage &ModelImage, const HRegionArray &ComponentRegions, long VariationRow, long VariationColumn, double VariationAngle, double AngleStart, double AngleExtent, long ContrastLowComp, long ContrastHighComp, long MinSizeComp, long MinContrastComp, double MinScoreComp, long NumLevelsComp, double AngleStepComp, const char *OptimizationComp, const char *MetricComp, const char *PregenerationComp);
  // Prepare a component model for matching based on trained components.
  virtual HTuple CreateTrainedComponentModel(const HTuple &ComponentTrainingID, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &MinContrastComp, const HTuple &MinScoreComp, const HTuple &NumLevelsComp, const HTuple &AngleStepComp, const HTuple &OptimizationComp, const HTuple &MetricComp, const HTuple &PregenerationComp);
  virtual HTuple CreateTrainedComponentModel(long ComponentTrainingID, double AngleStart, double AngleExtent, long MinContrastComp, double MinScoreComp, long NumLevelsComp, double AngleStepComp, const char *OptimizationComp, const char *MetricComp, const char *PregenerationComp);
};

#endif
