/*****************************************************************************
 * HFramegrabber.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HFramegrabber
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.5 $
 * $Date: 2003/10/02 09:54:53 $
 *
 */

#ifndef H_FRAMEGRABBER_H
#define H_FRAMEGRABBER_H

class LIntExport HFramegrabber: public HRootObject {
public:
  HFramegrabber(void) {id = -1;}
  HFramegrabber(
      const HTuple &Name, 
      const HTuple &HorizontalResolution = 1, const HTuple &VerticalResolution = 1,
      const HTuple &ImageWidth = 0,           const HTuple &ImageHeight = 0,
      const HTuple &StartRow = 0,             const HTuple &StartColumn = 0,
      const HTuple &Field = "default",
      const HTuple &BitsPerChannel = -1, 
      const HTuple &ColorSpace = "default",
      const HTuple &Gain = -1,
      const HTuple &ExternalTrigger = "default", 
      const HTuple &CameraType = "default",
      const HTuple &Device = "default",
      const HTuple &Port = -1,
      const HTuple &LineIn = -1);
  HFramegrabber(
      const char *Name, 
      long HorizontalResolution = 1, long VerticalResolution = 1,
      long ImageWidth = 0,           long ImageHeight = 0,
      long StartRow = 0,             long StartColumn = 0,
      const char *Field = "default",
      long BitsPerChannel = -1, 
      const char *ColorSpace = "default",
      double Gain = -1,
      const char *ExternalTrigger = "default", 
      const char *CameraType = "default",
      const char *Device = "default",
      long Port = -1,
      long LineIn = -1);

  ~HFramegrabber(void);
private:
  // Copying and Assignment not allowed
  HFramegrabber(const HFramegrabber &inst);
  HFramegrabber &operator= (const HFramegrabber &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HFramegrabber";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  // Handle
  long  id;
public:
  // Get specific parameters for a frame grabber.
  virtual HTuple GetFramegrabberParam(const HTuple &Param) const;
  virtual HTuple GetFramegrabberParam(const char *Param) const;
  // Set specific parameters for a frame grabber.
  virtual void SetFramegrabberParam(const HTuple &Param, const HTuple &Value) const;
  // Grab and segment an image from the specified frame grabber and
  // start the asynchronous grab of the next image.
  virtual HRegionArray GrabRegionAsync(const HTuple &MaxDelay) const;
  virtual HRegionArray GrabRegionAsync(double MaxDelay) const;
  // Grab and segment an image from the specified frame grabber.
  virtual HRegionArray GrabRegion() const;
  // Grab of an image from the specified frame grabber and start of the 
  // asynchronous grab of the next image.
  virtual HImage GrabImageAsync(const HTuple &MaxDelay) const;
  virtual HImage GrabImageAsync(double MaxDelay) const;
  // Start an asynchronous grab of an image from the specified frame grabber.
  virtual void GrabImageStart(const HTuple &MaxDelay) const;
  virtual void GrabImageStart(double MaxDelay) const;
  // Grab an image from the specified frame grabber.
  virtual HImage GrabImage() const;
  // Open and configure a frame grabber.
  virtual void OpenFramegrabber(const HTuple &Name, const HTuple &HorizontalResolution, const HTuple &VerticalResolution, const HTuple &ImageWidth, const HTuple &ImageHeight, const HTuple &StartRow, const HTuple &StartColumn, const HTuple &Field, const HTuple &BitsPerChannel, const HTuple &ColorSpace, const HTuple &Gain, const HTuple &ExternalTrigger, const HTuple &CameraType, const HTuple &Device, const HTuple &Port, const HTuple &LineIn);
  virtual void OpenFramegrabber(const char *Name, long HorizontalResolution, long VerticalResolution, long ImageWidth, long ImageHeight, long StartRow, long StartColumn, const char *Field, long BitsPerChannel, const char *ColorSpace, double Gain, const char *ExternalTrigger, const char *CameraType, const char *Device, long Port, long LineIn);
  // Query frame grabber lut.
  virtual HTuple GetFramegrabberLut(HTuple *ImageGreen, HTuple *ImageBlue) const;
  // Set frame grabber lut.
  virtual void SetFramegrabberLut(const HTuple &ImageRed, const HTuple &ImageGreen, const HTuple &ImageBlue) const;
};

#endif
