/*****************************************************************************
 * HImage.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Types for Halcon-images
 *
 * (c) 1996-2000 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.16 $
 * $Date: 2001/12/06 09:53:07 $
 *
 */

#ifndef H_IMAGE_H
#define H_IMAGE_H


/****************************************************************************/
/*                               HImage                                     */
/****************************************************************************/
class HImageArray;
class LIntExport HImage: public HObject {
  friend class HImageArray;
public:
  HImage(void) :     HObject(),init(FALSE),privat(FALSE),
    width(0),height(0),type(0),ptr(0) {num_images++;}
  HImage(Hkey key) : HObject(key),init(FALSE),privat(FALSE),
    width(0),height(0),type(0),ptr(0) {num_images++;}
  HImage(const Hobject &obj);
  HImage(const char *file);
  HImage(const HImage &r);
  HImage(const HImageArray &arr);
  HImage(int width, int height, const char *type);
  HImage(void *ptr, int width, int height, const char *type);
  HImage(void *ptr, void *ptrfreecall, int loc_width, int loc_height,
	 const char *type);
  virtual ~HImage(void) {num_images--;}
  HImage &operator = (const HImage &image);
  HImage &operator = (const HImageArray &arr);
  
  // Tools
  virtual const char   *Creator(void) const;
	        long   NumImageRefs(void) const;
                HBool  Shared(void) const {return NumImageRefs() != 1;}
  // Name der Halcon-Klasse -> get_obj_class
  virtual const char   *HClassName(void) const;
  virtual const char   *InstClassName(void) const;
  virtual const char   *ClassName(void) const;
  virtual       INT4   ClassDBID(void) const {return IMAGE_ID;}

  virtual       void   CheckMyObjClass(void) const;
  virtual const char   *PixType(void) const;
          int          Width(void) const;
          int          Height(void) const;
  // Pixelzugriff
          HPixVal GetPixVal(int x, int y) const;
          HPixVal GetPixVal(long k) const;
  virtual void    SetPixVal(int x, int y, const HPixVal &val);
  virtual void    SetPixVal(long k, const HPixVal &val);
  
  virtual void    Transform(const HAffineTrans &transform, int mode);
  virtual void    Display(const HWindow &w) const;
  
  // Infix-Operationen
  HImage  operator & (const HRegion &reg) const;
  HImage  operator + (const HImage &add) const;
  HImage  operator - (const HImage &sub) const;
  HImage  operator * (const HImage &mult) const;
  HImage  operator - (void) const;
  HImage  operator + (double add) const;
  HImage  operator - (double sub) const;
  HImage  operator * (double mult) const;
  HImage  operator / (double div) const;
  HRegion operator >= (const HImage &image) const;
  HRegion operator <= (const HImage &image) const;
  HRegion operator >= (double thresh) const;
  HRegion operator <= (double thresh) const;
  HRegion operator == (double thresh) const;
  HRegion operator != (double thresh) const;
 
  // Klassenoperationen
  static long NumInst(void) { return num_images; }
  static long NumData(void);
protected:
  void   CopyConvertedImage(const HImage &in,const char *newType);
  void   Init(void);      // width, height zur Verfuegung stellen
  void   Private(void);   // Private Matrix erzwingen
  HBool  init;            // Init() wurde ausgefuehrt
  HBool  privat;          // Init() und Private() wurden ausgefuehrt
  int    word_length;     // 1,2 oder 4, je nach type
  int    width, height;   // Bildgroesse; gesetzt durch Init()
  int    type;            // Halcon-Pixeltyp fuer switch
  char   type_str[20];    // Halcon-Pixeltyp als String
  void   *ptr;            // Zeiger auf Matrix; gesetzt durch Private()
  void   SetInitial(void);// Ruecksetzen fuer Konstruktoren etc.
private:
  static long num_images;
public:
  
  // Find regions that might contain a 2D bar code.
  virtual HRegionArray Find2dBarCode(const HTuple &BarCodeDescr, const HTuple &GenParamNames, const HTuple &GenParamValues, HTuple *CodeRegDescr) const;
  // Extract the widths of the elements inside a bar code region.
  virtual HTuple Get1dBarCodeScanline(const HTuple &BarCodeDescr, const HTuple &GenericName, const HTuple &GenericValue, const HTuple &Orientation, const HTuple &StopIfFound, HTuple *ScanlineNumPoints, HTuple *ScanlineDecoded, HTuple *ScanlinePointsRow, HTuple *ScanlinePointsColumn) const;
  // Extract the widths of the elements inside a bar code region.
  virtual HTuple Get1dBarCode(const HTuple &BarCodeDescr, const HTuple &GenericName, const HTuple &GenericValue, const HTuple &Orientation) const;
  // Look for multiple bar code regions in an image.
  virtual HRegionArray Find1dBarCodeRegion(const HTuple &BarCodeDescr, const HTuple &GenericName, const HTuple &GenericValue, HTuple *Orientation) const;
  // Look for one bar code in an image.
  virtual HRegion Find1dBarCodeScanline(const HTuple &BarCodeDescr, const HTuple &GenericName, const HTuple &GenericValue, const HTuple &StopIfFound, HTuple *BarcodeFound, HTuple *BarCodeElements, HTuple *Orientation, HTuple *ScanlineNumPoints, HTuple *ScanlineDecoded, HTuple *ScanlinePointsRow, HTuple *ScanlinePointsColumn) const;
  // Look for one bar code in an image.
  virtual HRegion Find1dBarCode(const HTuple &BarCodeDescr, const HTuple &GenericName, const HTuple &GenericValue, HTuple *BarcodeFound, HTuple *BarCodeElements, HTuple *Orientation) const;
  // Return the estimated background image.
  static HImage GiveBgEsti(const HTuple &BgEstiHandle);
  static HImage GiveBgEsti(long BgEstiHandle);
  // Change the estimated background image.
  virtual void UpdateBgEsti(const HRegion &UpDateRegion, const HTuple &BgEstiHandle) const;
  virtual void UpdateBgEsti(const HRegion &UpDateRegion, long BgEstiHandle) const;
  // Estimate the background and return the foreground region.
  virtual HRegion RunBgEsti(const HTuple &BgEstiHandle) const;
  virtual HRegion RunBgEsti(long BgEstiHandle) const;
  // Generate and initialize a data set for the background estimation.
  virtual long CreateBgEsti(const HTuple &Syspar1, const HTuple &Syspar2, const HTuple &GainMode, const HTuple &Gain1, const HTuple &Gain2, const HTuple &AdaptMode, const HTuple &MinDiff, const HTuple &StatNum, const HTuple &ConfidenceC, const HTuple &TimeC) const;
  virtual long CreateBgEsti(double Syspar1, double Syspar2, const char *GainMode, double Gain1, double Gain2, const char *AdaptMode, double MinDiff, long StatNum, double ConfidenceC, double TimeC) const;
  // Grab of an image from the specified frame grabber and start of the 
  // asynchronous grab of the next image.
  static HImage GrabImageAsync(const HTuple &FGHandle, const HTuple &MaxDelay);
  static HImage GrabImageAsync(long FGHandle, double MaxDelay);
  // Grab an image from the specified frame grabber.
  static HImage GrabImage(const HTuple &FGHandle);
  static HImage GrabImage(long FGHandle);
  // Convert a multichannel image into One-channel images
  virtual HImageArray ImageToChannels(void) const;
  // Convert 7 images into a seven-channel image.
  virtual HImage Compose7(const HImage &Image2, const HImage &Image3, const HImage &Image4, const HImage &Image5, const HImage &Image6, const HImage &Image7) const;
  virtual HImage Compose7(const HImageArray &Image2, const HImageArray &Image3, const HImageArray &Image4, const HImageArray &Image5, const HImageArray &Image6, const HImageArray &Image7) const;
  // Convert 6 images into a six-channel image.
  virtual HImage Compose6(const HImage &Image2, const HImage &Image3, const HImage &Image4, const HImage &Image5, const HImage &Image6) const;
  virtual HImage Compose6(const HImageArray &Image2, const HImageArray &Image3, const HImageArray &Image4, const HImageArray &Image5, const HImageArray &Image6) const;
  // Convert 5 images into a five-channel image.
  virtual HImage Compose5(const HImage &Image2, const HImage &Image3, const HImage &Image4, const HImage &Image5) const;
  virtual HImage Compose5(const HImageArray &Image2, const HImageArray &Image3, const HImageArray &Image4, const HImageArray &Image5) const;
  // Convert 4 images into a four-channel image.
  virtual HImage Compose4(const HImage &Image2, const HImage &Image3, const HImage &Image4) const;
  virtual HImage Compose4(const HImageArray &Image2, const HImageArray &Image3, const HImageArray &Image4) const;
  // Convert 3 images into a three-channel image.
  virtual HImage Compose3(const HImage &Image2, const HImage &Image3) const;
  virtual HImage Compose3(const HImageArray &Image2, const HImageArray &Image3) const;
  // Convert two images into a two-channel image.
  virtual HImage Compose2(const HImage &Image2) const;
  virtual HImage Compose2(const HImageArray &Image2) const;
  // Convert a seven-channel image into seven images.
  virtual HImage Decompose7(HImage *Image2, HImage *Image3, HImage *Image4, HImage *Image5, HImage *Image6, HImage *Image7) const;
  // Convert a six-channel image into six images.
  virtual HImage Decompose6(HImage *Image2, HImage *Image3, HImage *Image4, HImage *Image5, HImage *Image6) const;
  // Convert a five-channel image into five images.
  virtual HImage Decompose5(HImage *Image2, HImage *Image3, HImage *Image4, HImage *Image5) const;
  // Convert a four-channel image into four images.
  virtual HImage Decompose4(HImage *Image2, HImage *Image3, HImage *Image4) const;
  // Convert a three-channel image into three images.
  virtual HImage Decompose3(HImage *Image2, HImage *Image3) const;
  // Convert a two-channel image into two images.
  virtual HImage Decompose2(HImage *Image2) const;
  // Count channels of image.
  virtual long CountChannels(void) const;
  // Append additional matrices (channels) to the image.
  virtual HImage AppendChannel(const HImage &Image) const;
  // Access a channel of a multichannel image.
  virtual HImage AccessChannel(const HTuple &Channel) const;
  virtual HImage AccessChannel(long Channel) const;
  // Tile multiple image objects into a large image with explicit
  // positioning information.
  virtual HImage TileImagesOffset(const HTuple &OffsetRow, const HTuple &OffsetCol, const HTuple &Row1, const HTuple &Col1, const HTuple &Row2, const HTuple &Col2, const HTuple &Width, const HTuple &Height) const;
  virtual HImage TileImagesOffset(long OffsetRow, long OffsetCol, long Row1, long Col1, long Row2, long Col2, long Width, long Height) const;
  // Tile multiple images into a large image.
  virtual HImage TileChannels(const HTuple &NumColumns, const HTuple &TileOrder) const;
  virtual HImage TileChannels(long NumColumns, const char *TileOrder) const;
  // Cut out of defined gray values.
  virtual HImage CropDomain(void) const;
  // Cut out a rectangular image area.
  virtual HImage CropRectangle1(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2) const;
  virtual HImage CropRectangle1(long Row1, long Column1, long Row2, long Column2) const;
  // Cut out a rectangular image area.
  virtual HImage CropPart(const HTuple &Row, const HTuple &Column, const HTuple &Width, const HTuple &Height) const;
  virtual HImage CropPart(long Row, long Column, long Width, long Height) const;
  // Change image size.
  virtual HImage ChangeFormat(const HTuple &Width, const HTuple &Height) const;
  virtual HImage ChangeFormat(long Width, long Height) const;
  // Change definition domain of an image.
  virtual HImage ChangeDomain(const HRegion &NewDomain) const;
  // Reduce the domain of an image to a rectangle.
  virtual HImage Rectangle1Domain(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2) const;
  virtual HImage Rectangle1Domain(long Row1, long Column1, long Row2, long Column2) const;
  // Reduce the domain of an image.
  virtual HImage ReduceDomain(const HRegion &Region) const;
  // Expand the domain of an image to maximum.
  virtual HImage FullDomain(void) const;
  // Get the domain of an image.
  virtual HRegion GetDomain(void) const;
  // Detect and read 2D data code symbols in an image or
  // train the 2D data code model.
  virtual HXLDContArray FindDataCode2d(const HTuple &DataCodeHandle, const HTuple &GenParamNames, const HTuple &GenParamValues, HTuple *ResultHandles, HTuple *DecodedDataStrings) const;
  virtual HXLDContArray FindDataCode2d(long DataCodeHandle, const char *GenParamNames, long GenParamValues, HTuple *ResultHandles, HTuple *DecodedDataStrings) const;
  // Write images in graphic formats.
  virtual void WriteImage(const HTuple &Format, const HTuple &FillColor, const HTuple &FileName) const;
  virtual void WriteImage(const char *Format, long FillColor, const char *FileName) const;
  // Read images.
  static HImage ReadSequence(const HTuple &HeaderSize, const HTuple &SourceWidth, const HTuple &SourceHeight, const HTuple &StartRow, const HTuple &StartColumn, const HTuple &DestWidth, const HTuple &DestHeight, const HTuple &PixelType, const HTuple &BitOrder, const HTuple &ByteOrder, const HTuple &Pad, const HTuple &Index, const HTuple &FileName);
  static HImage ReadSequence(long HeaderSize, long SourceWidth, long SourceHeight, long StartRow, long StartColumn, long DestWidth, long DestHeight, const char *PixelType, const char *BitOrder, const char *ByteOrder, const char *Pad, long Index, const char *FileName);
  // Read an image with different file formats.
  static HImage ReadImage(const HTuple &FileName);
  static HImage ReadImage(const char *FileName);
  // Apply a projective transformation to an image and specify the output
  // image size.
  virtual HImage ProjectiveTransImageSize(const HTuple &HomMat2D, const HTuple &Interpolation, const HTuple &Width, const HTuple &Height, const HTuple &TransformRegion) const;
  virtual HImage ProjectiveTransImageSize(double HomMat2D, const char *Interpolation, long Width, long Height, const char *TransformRegion) const;
  // Apply a projective transformation to an image.
  virtual HImage ProjectiveTransImage(const HTuple &HomMat2D, const HTuple &Interpolation, const HTuple &AdaptImageSize, const HTuple &TransformRegion) const;
  virtual HImage ProjectiveTransImage(double HomMat2D, const char *Interpolation, const char *AdaptImageSize, const char *TransformRegion) const;
  // Apply an arbitrary affine 2D transformation to an image and specify the
  // output image size.
  virtual HImage AffineTransImageSize(const HTuple &HomMat2D, const HTuple &Interpolation, const HTuple &Width, const HTuple &Height) const;
  virtual HImage AffineTransImageSize(double HomMat2D, const char *Interpolation, long Width, long Height) const;
  // Apply an arbitrary affine 2D transformation to images.
  virtual HImage AffineTransImage(const HTuple &HomMat2D, const HTuple &Interpolation, const HTuple &AdaptImageSize) const;
  virtual HImage AffineTransImage(double HomMat2D, const char *Interpolation, const char *AdaptImageSize) const;
  // Zoom an image by a given factor.
  virtual HImage ZoomImageFactor(const HTuple &ScaleWidth, const HTuple &ScaleHeight, const HTuple &Interpolation) const;
  virtual HImage ZoomImageFactor(double ScaleWidth, double ScaleHeight, const char *Interpolation) const;
  // Zoom an image to a given size.
  virtual HImage ZoomImageSize(const HTuple &Width, const HTuple &Height, const HTuple &Interpolation) const;
  virtual HImage ZoomImageSize(long Width, long Height, const char *Interpolation) const;
  // Mirror an image.
  virtual HImage MirrorImage(const HTuple &Mode) const;
  virtual HImage MirrorImage(const char *Mode) const;
  // Rotate an image about its center.
  virtual HImage RotateImage(const HTuple &Phi, const HTuple &Interpolation) const;
  virtual HImage RotateImage(double Phi, const char *Interpolation) const;
  // Transform an image in polar coordinates back to cartesian coordinates
  virtual HImage PolarTransImageInv(const HTuple &Row, const HTuple &Column, const HTuple &AngleStart, const HTuple &AngleEnd, const HTuple &RadiusStart, const HTuple &RadiusEnd, const HTuple &Width, const HTuple &Height, const HTuple &Interpolation) const;
  virtual HImage PolarTransImageInv(double Row, double Column, double AngleStart, double AngleEnd, double RadiusStart, double RadiusEnd, long Width, long Height, const char *Interpolation) const;
  // Transform an annular arc in an image to polar coordinates.
  virtual HImage PolarTransImageExt(const HTuple &Row, const HTuple &Column, const HTuple &AngleStart, const HTuple &AngleEnd, const HTuple &RadiusStart, const HTuple &RadiusEnd, const HTuple &Width, const HTuple &Height, const HTuple &Interpolation) const;
  virtual HImage PolarTransImageExt(double Row, double Column, double AngleStart, double AngleEnd, double RadiusStart, double RadiusEnd, long Width, long Height, const char *Interpolation) const;
  // Transform an image to polar coordinates
  virtual HImage PolarTransImage(const HTuple &Row, const HTuple &Column, const HTuple &Width, const HTuple &Height) const;
  virtual HImage PolarTransImage(long Row, long Column, long Width, long Height) const;
  // Approximate an affine map from a displacement vector field.
  virtual HTuple DvfToHomMat2d(void) const;
  // Calculate the square root of an image.
  virtual HImage SqrtImage(void) const;
  // Subtract two images.
  virtual HImage SubImage(const HImage &ImageSubtrahend, const HTuple &Mult, const HTuple &Add) const;
  virtual HImage SubImage(const HImage &ImageSubtrahend, double Mult, double Add) const;
  virtual HImage SubImage(const HImageArray &ImageSubtrahend, const HTuple &Mult, const HTuple &Add) const;
  virtual HImage SubImage(const HImageArray &ImageSubtrahend, double Mult, double Add) const;
  // Scale the gray values of an image.
  virtual HImage ScaleImage(const HTuple &Mult, const HTuple &Add) const;
  virtual HImage ScaleImage(double Mult, double Add) const;
  // Divide two images.
  virtual HImage DivImage(const HImage &Image2, const HTuple &Mult, const HTuple &Add) const;
  virtual HImage DivImage(const HImage &Image2, double Mult, double Add) const;
  virtual HImage DivImage(const HImageArray &Image2, const HTuple &Mult, const HTuple &Add) const;
  virtual HImage DivImage(const HImageArray &Image2, double Mult, double Add) const;
  // Multiply two images.
  virtual HImage MultImage(const HImage &Image2, const HTuple &Mult, const HTuple &Add) const;
  virtual HImage MultImage(const HImage &Image2, double Mult, double Add) const;
  virtual HImage MultImage(const HImageArray &Image2, const HTuple &Mult, const HTuple &Add) const;
  virtual HImage MultImage(const HImageArray &Image2, double Mult, double Add) const;
  // Add two images.
  virtual HImage AddImage(const HImage &Image2, const HTuple &Mult, const HTuple &Add) const;
  virtual HImage AddImage(const HImage &Image2, double Mult, double Add) const;
  virtual HImage AddImage(const HImageArray &Image2, const HTuple &Mult, const HTuple &Add) const;
  virtual HImage AddImage(const HImageArray &Image2, double Mult, double Add) const;
  // Calculate the absolute value (modulus) of an image.
  virtual HImage AbsImage(void) const;
  // Calculate the minimum of two images pixel by pixel.
  virtual HImage MinImage(const HImage &Image2) const;
  virtual HImage MinImage(const HImageArray &Image2) const;
  // Calculate the maximum of two images pixel by pixel.
  virtual HImage MaxImage(const HImage &Image2) const;
  virtual HImage MaxImage(const HImageArray &Image2) const;
  // Invert an image.
  virtual HImage InvertImage(void) const;
  // Logical ``AND'' of each pixel using a bit mask.
  virtual HImage BitMask(const HTuple &BitMask) const;
  virtual HImage BitMask(long BitMask) const;
  // Extract a bit from the pixels.
  virtual HImage BitSlice(const HTuple &Bit) const;
  virtual HImage BitSlice(long Bit) const;
  // Right shift of all pixels of the image.
  virtual HImage BitRshift(const HTuple &Shift) const;
  virtual HImage BitRshift(long Shift) const;
  // Left shift of all pixels of the image.
  virtual HImage BitLshift(const HTuple &Shift) const;
  virtual HImage BitLshift(long Shift) const;
  // Complement all bits of the pixels.
  virtual HImage BitNot(void) const;
  // Bit-by-bit XOR of all pixels of the input images.
  virtual HImage BitXor(const HImage &Image2) const;
  virtual HImage BitXor(const HImageArray &Image2) const;
  // Bit-by-bit OR of all pixels of the input images.
  virtual HImage BitOr(const HImage &Image2) const;
  virtual HImage BitOr(const HImageArray &Image2) const;
  // Bit-by-bit AND of all pixels of the input images.
  virtual HImage BitAnd(const HImage &Image2) const;
  virtual HImage BitAnd(const HImageArray &Image2) const;
  // Convert a single-channel color filter array image into an RGB image.
  virtual HImage CfaToRgb(const HTuple &CFAType, const HTuple &Interpolation) const;
  virtual HImage CfaToRgb(const char *CFAType, const char *Interpolation) const;
  // Transform an RGB image into a gray scale image.
  virtual HImage Rgb1ToGray(void) const;
  // Transform an RGB image to a gray scale image.
  virtual HImage Rgb3ToGray(const HImage &ImageGreen, const HImage &ImageBlue) const;
  virtual HImage Rgb3ToGray(const HImageArray &ImageGreen, const HImageArray &ImageBlue) const;
  // Transform an image from the RGB color space to an arbitrary color space.
  virtual HImage TransFromRgb(const HImage &ImageGreen, const HImage &ImageBlue, HImage *ImageResult2, HImage *ImageResult3, const HTuple &ColorSpace) const;
  virtual HImage TransFromRgb(const HImage &ImageGreen, const HImage &ImageBlue, HImage *ImageResult2, HImage *ImageResult3, const char *ColorSpace) const;
  virtual HImage TransFromRgb(const HImageArray &ImageGreen, const HImageArray &ImageBlue, HImage *ImageResult2, HImage *ImageResult3, const HTuple &ColorSpace) const;
  virtual HImage TransFromRgb(const HImageArray &ImageGreen, const HImageArray &ImageBlue, HImage *ImageResult2, HImage *ImageResult3, const char *ColorSpace) const;
  // Transform an image from an arbitrary color space to the RGB color space.
  virtual HImage TransToRgb(const HImage &ImageInput2, const HImage &ImageInput3, HImage *ImageGreen, HImage *ImageBlue, const HTuple &ColorSpace) const;
  virtual HImage TransToRgb(const HImage &ImageInput2, const HImage &ImageInput3, HImage *ImageGreen, HImage *ImageBlue, const char *ColorSpace) const;
  virtual HImage TransToRgb(const HImageArray &ImageInput2, const HImageArray &ImageInput3, HImage *ImageGreen, HImage *ImageBlue, const HTuple &ColorSpace) const;
  virtual HImage TransToRgb(const HImageArray &ImageInput2, const HImageArray &ImageInput3, HImage *ImageGreen, HImage *ImageBlue, const char *ColorSpace) const;
  // Store a filter mask in the spatial domain as a real-image.
  static HImage GenFilterMask(const HTuple &FilterMask, const HTuple &Scale, const HTuple &Width, const HTuple &Height);
  // Generate a Gaussian filter in the frequency domain.
  static HImage GenGaussFilter(const HTuple &Sigma1, const HTuple &Sigma2, const HTuple &Phi, const HTuple &Norm, const HTuple &Mode, const HTuple &Width, const HTuple &Height);
  static HImage GenGaussFilter(double Sigma1, double Sigma2, double Phi, const char *Norm, const char *Mode, long Width, long Height);
  // Generate a derivative filter in the frequency domain.
  static HImage GenDerivativeFilter(const HTuple &Derivative, const HTuple &Exponent, const HTuple &Norm, const HTuple &Mode, const HTuple &Width, const HTuple &Height);
  static HImage GenDerivativeFilter(const char *Derivative, long Exponent, const char *Norm, const char *Mode, long Width, long Height);
  // Generate a bandpass filter with Gaussian or sinusoidal shape.
  static HImage GenStdBandpass(const HTuple &Frequency, const HTuple &Sigma, const HTuple &Type, const HTuple &Norm, const HTuple &Mode, const HTuple &Width, const HTuple &Height);
  static HImage GenStdBandpass(double Frequency, double Sigma, const char *Type, const char *Norm, const char *Mode, long Width, long Height);
  // Generate a bandpass filter with sinusoidal shape.
  static HImage GenSinBandpass(const HTuple &Frequency, const HTuple &Norm, const HTuple &Mode, const HTuple &Width, const HTuple &Height);
  static HImage GenSinBandpass(double Frequency, const char *Norm, const char *Mode, long Width, long Height);
  // Generate an ideal band filter.
  static HImage GenBandfilter(const HTuple &MinFrequency, const HTuple &MaxFrequency, const HTuple &Norm, const HTuple &Mode, const HTuple &Width, const HTuple &Height);
  static HImage GenBandfilter(double MinFrequency, double MaxFrequency, const char *Norm, const char *Mode, long Width, long Height);
  // Generate an ideal bandpass filter.
  static HImage GenBandpass(const HTuple &MinFrequency, const HTuple &MaxFrequency, const HTuple &Norm, const HTuple &Mode, const HTuple &Width, const HTuple &Height);
  static HImage GenBandpass(double MinFrequency, double MaxFrequency, const char *Norm, const char *Mode, long Width, long Height);
  // Generate an ideal lowpass filter.
  static HImage GenLowpass(const HTuple &Frequency, const HTuple &Norm, const HTuple &Mode, const HTuple &Width, const HTuple &Height);
  static HImage GenLowpass(double Frequency, const char *Norm, const char *Mode, long Width, long Height);
  // Generate an ideal highpass filter.
  static HImage GenHighpass(const HTuple &Frequency, const HTuple &Norm, const HTuple &Mode, const HTuple &Width, const HTuple &Height);
  static HImage GenHighpass(double Frequency, const char *Norm, const char *Mode, long Width, long Height);
  // Return the power spectrum of a complex image.
  virtual HImage PowerLn(void) const;
  // Return the power spectrum of a complex image.
  virtual HImage PowerReal(void) const;
  // Return the power spectrum of a complex image.
  virtual HImage PowerByte(void) const;
  // Return the phase of a complex image in degrees.
  virtual HImage PhaseDeg(void) const;
  // Return the phase of a complex image in radians.
  virtual HImage PhaseRad(void) const;
  // Calculate the energy of a two-channel image.
  virtual HImage EnergyGabor(const HImage &ImageHilbert) const;
  virtual HImage EnergyGabor(const HImageArray &ImageHilbert) const;
  // Convolve an image with a Gabor filter in the frequency domain.
  virtual HImage ConvolGabor(const HImage &GaborFilter, HImage *ImageResultHilbert) const;
  // Generate a Gabor filter.
  static HImage GenGabor(const HTuple &Angle, const HTuple &Frequency, const HTuple &Bandwidth, const HTuple &Orientation, const HTuple &Norm, const HTuple &Mode, const HTuple &Width, const HTuple &Height);
  static HImage GenGabor(double Angle, double Frequency, double Bandwidth, double Orientation, const char *Norm, const char *Mode, long Width, long Height);
  // Compute the correlation of two images in the frequency domain.
  virtual HImage CorrelationFft(const HImage &ImageFFT2) const;
  virtual HImage CorrelationFft(const HImageArray &ImageFFT2) const;
  // Convolve an image with a filter in the frequency domain.
  virtual HImage ConvolFft(const HImage &ImageFilter) const;
  // Compute the real-valued fast Fourier transform of an image.
  virtual HImage RftGeneric(const HTuple &Direction, const HTuple &Norm, const HTuple &ResultType, const HTuple &Width) const;
  virtual HImage RftGeneric(const char *Direction, const char *Norm, const char *ResultType, long Width) const;
  // Compute the inverse fast Fourier transform of an image.
  virtual HImage FftImageInv(void) const;
  // Compute the fast Fourier transform of an image.
  virtual HImage FftImage(void) const;
  // Compute the fast Fourier transform of an image.
  virtual HImage FftGeneric(const HTuple &Direction, const HTuple &Exponent, const HTuple &Norm, const HTuple &Mode, const HTuple &ResultType) const;
  virtual HImage FftGeneric(const char *Direction, long Exponent, const char *Norm, const char *Mode, const char *ResultType) const;
  // Smooth an image using recursive filters.
  virtual HImage SmoothImage(const HTuple &Filter, const HTuple &Alpha) const;
  virtual HImage SmoothImage(const char *Filter, double Alpha) const;
  // Non-linear smoothing with the sigma filter.
  virtual HImage SigmaImage(const HTuple &MaskHeight, const HTuple &MaskWidth, const HTuple &Sigma) const;
  virtual HImage SigmaImage(long MaskHeight, long MaskWidth, long Sigma) const;
  // Calculate the average of maximum and minimum inside any mask.
  virtual HImage MidrangeImage(const HRegion &Mask, const HTuple &Margin) const;
  // Smooth an image with an arbitrary rank mask.
  virtual HImage TrimmedMean(const HRegion &Mask, const HTuple &Number, const HTuple &Margin) const;
  // Separated median filtering with rectangle masks.
  virtual HImage MedianSeparate(const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &Margin) const;
  // Median filtering with different rank masks.
  virtual HImage MedianImage(const HTuple &MaskType, const HTuple &Radius, const HTuple &Margin) const;
  // Weighted median filtering with different rank masks.
  virtual HImage MedianWeighted(const HTuple &MaskType, const HTuple &MaskSize) const;
  virtual HImage MedianWeighted(const char *MaskType, long MaskSize) const;
  // Smooth an image with an arbitrary rank mask.
  virtual HImage RankImage(const HRegion &Mask, const HTuple &Rank, const HTuple &Margin) const;
  // Opening, Median and Closing with circle or rectangle mask.
  virtual HImage DualRank(const HTuple &MaskType, const HTuple &Radius, const HTuple &ModePercent, const HTuple &Margin) const;
  // Smooth by averaging.
  virtual HImage MeanImage(const HTuple &MaskWidth, const HTuple &MaskHeight) const;
  virtual HImage MeanImage(long MaskWidth, long MaskHeight) const;
  // Smooth using discrete gauss functions.
  virtual HImage GaussImage(const HTuple &Size) const;
  virtual HImage GaussImage(long Size) const;
  // Smooth an image in the spatial domain to suppress noise.
  virtual HImage EliminateMinMax(const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &Gap, const HTuple &Mode) const;
  virtual HImage EliminateMinMax(long MaskWidth, long MaskHeight, double Gap, long Mode) const;
  // Smooth an image by edge-preserving anisotropic diffusion.
  virtual HImage AnisotropeDiff(const HTuple &Percent, const HTuple &Mode, const HTuple &Iteration, const HTuple &NeighborhoodType) const;
  virtual HImage AnisotropeDiff(long Percent, long Mode, long Iteration, long NeighborhoodType) const;
  // Interpolate 2 video half images.
  virtual HImage FillInterlace(const HTuple &Mode) const;
  virtual HImage FillInterlace(const char *Mode) const;
  // Average gray values over several channels.
  virtual HImage MeanN(void) const;
  // Replace values outside of thresholds with average value.
  virtual HImage EliminateSp(const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &MinThresh, const HTuple &MaxThresh) const;
  virtual HImage EliminateSp(long MaskWidth, long MaskHeight, long MinThresh, long MaxThresh) const;
  // Suppress salt and pepper noise.
  virtual HImage MeanSp(const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &MinThresh, const HTuple &MaxThresh) const;
  virtual HImage MeanSp(long MaskWidth, long MaskHeight, long MinThresh, long MaxThresh) const;
  // Detect edges (amplitude) using the Sobel operator.
  virtual HImage SobelAmp(const HTuple &FilterType, const HTuple &Size) const;
  virtual HImage SobelAmp(const char *FilterType, long Size) const;
  // Detect edges (amplitude and direction) using the Sobel operator.
  virtual HImage SobelDir(HImage *EdgeDirection, const HTuple &FilterType, const HTuple &Size) const;
  virtual HImage SobelDir(HImage *EdgeDirection, const char *FilterType, long Size) const;
  // Detect edges using the Roberts filter.
  virtual HImage Roberts(const HTuple &FilterType) const;
  virtual HImage Roberts(const char *FilterType) const;
  // Calculate the Laplace operator by using finite differences.
  virtual HImage Laplace(const HTuple &FilterType, const HTuple &Size, const HTuple &NeighbourhoodType) const;
  virtual HImage Laplace(const char *FilterType, long Size, const char *NeighbourhoodType) const;
  // Extract high frequency components from an image.
  virtual HImage HighpassImage(const HTuple &Width, const HTuple &Height) const;
  virtual HImage HighpassImage(long Width, long Height) const;
  // Extract subpixel precise color edges using Deriche, Shen, or Canny filters.
  virtual HXLDContArray EdgesColorSubPix(const HTuple &Filter, const HTuple &Alpha, const HTuple &Low, const HTuple &High) const;
  virtual HXLDContArray EdgesColorSubPix(const char *Filter, double Alpha, double Low, double High) const;
  // Extract color edges using Canny, Deriche, or Shen filters.
  virtual HImage EdgesColor(HImage *ImaDir, const HTuple &Filter, const HTuple &Alpha, const HTuple &NMS, const HTuple &Low, const HTuple &High) const;
  virtual HImage EdgesColor(HImage *ImaDir, const char *Filter, double Alpha, const char *NMS, long Low, long High) const;
  // Extract sub-pixel precise edges using Deriche, Lanser, Shen, or Canny
  // filters.
  virtual HXLDContArray EdgesSubPix(const HTuple &Filter, const HTuple &Alpha, const HTuple &Low, const HTuple &High) const;
  virtual HXLDContArray EdgesSubPix(const char *Filter, double Alpha, long Low, long High) const;
  // Extract edges using Deriche, Lanser, Shen, or Canny filters.
  virtual HImage EdgesImage(HImage *ImaDir, const HTuple &Filter, const HTuple &Alpha, const HTuple &NMS, const HTuple &Low, const HTuple &High) const;
  virtual HImage EdgesImage(HImage *ImaDir, const char *Filter, double Alpha, const char *NMS, long Low, long High) const;
  // Convolve an image with derivatives of the Gaussian.
  virtual HImage DerivateGauss(const HTuple &Sigma, const HTuple &Component) const;
  virtual HImage DerivateGauss(double Sigma, const char *Component) const;
  // LoG-Operator (Laplace of Gaussian).
  virtual HImage LaplaceOfGauss(const HTuple &Sigma) const;
  virtual HImage LaplaceOfGauss(double Sigma) const;
  // Approximate the LoG operator (Laplace of Gaussian).
  virtual HImage DiffOfGauss(const HTuple &Sigma, const HTuple &SigFactor) const;
  virtual HImage DiffOfGauss(double Sigma, double SigFactor) const;
  // Detect straight edge segments.
  virtual HTuple DetectEdgeSegments(const HTuple &SobelSize, const HTuple &MinAmplitude, const HTuple &MaxDistance, const HTuple &MinLength, HTuple *BeginCol, HTuple *EndRow, HTuple *EndCol) const;
  virtual HTuple DetectEdgeSegments(long SobelSize, long MinAmplitude, long MaxDistance, long MinLength, HTuple *BeginCol, HTuple *EndRow, HTuple *EndCol) const;
  // Detect edges (amplitude and direction) using the Robinson operator.
  virtual HImage RobinsonDir(HImage *ImageEdgeDir) const;
  // Detect edges (amplitude) using the Robinson operator.
  virtual HImage RobinsonAmp(void) const;
  // Detect edges (amplitude and direction) using the Kirsch operator.
  virtual HImage KirschDir(HImage *ImageEdgeDir) const;
  // Detect edges (amplitude) using the Kirsch operator.
  virtual HImage KirschAmp(void) const;
  // Detect edges (amplitude and direction) using the Frei-Chen operator.
  virtual HImage FreiDir(HImage *ImageEdgeDir) const;
  // Detect edges (amplitude) using the Frei-Chen operator.
  virtual HImage FreiAmp(void) const;
  // Detect edges (amplitude and direction) using the Prewitt operator.
  virtual HImage PrewittDir(HImage *ImageEdgeDir) const;
  // Detect edges (amplitude) using the Prewitt operator.
  virtual HImage PrewittAmp(void) const;
  // Edge extraction using bandpass filters.
  virtual HImage BandpassImage(const HTuple &FilterType) const;
  virtual HImage BandpassImage(const char *FilterType) const;
  // Detect color lines and their width.
  virtual HXLDContArray LinesColor(const HTuple &Sigma, const HTuple &Low, const HTuple &High, const HTuple &ExtractWidth, const HTuple &CompleteJunctions) const;
  virtual HXLDContArray LinesColor(double Sigma, double Low, double High, const char *ExtractWidth, const char *CompleteJunctions) const;
  // Detect lines and their width.
  virtual HXLDContArray LinesGauss(const HTuple &Sigma, const HTuple &Low, const HTuple &High, const HTuple &LightDark, const HTuple &ExtractWidth, const HTuple &CorrectPositions, const HTuple &CompleteJunctions) const;
  virtual HXLDContArray LinesGauss(double Sigma, double Low, double High, const char *LightDark, const char *ExtractWidth, const char *CorrectPositions, const char *CompleteJunctions) const;
  // Detection of lines using the facet model.
  virtual HXLDContArray LinesFacet(const HTuple &MaskSize, const HTuple &Low, const HTuple &High, const HTuple &LightDark) const;
  virtual HXLDContArray LinesFacet(long MaskSize, double Low, double High, const char *LightDark) const;
  // Calculating the Displacement vector field by correlation methods.
  virtual HImage OpticalFlowMatch(const HImage &Image2, const HTuple &Mode, const HTuple &Threshold, const HTuple &Step, const HTuple &SizeWindow, const HTuple &SizeSearch, const HTuple &Weights) const;
  virtual HImage OpticalFlowMatch(const HImage &Image2, const char *Mode, long Threshold, long Step, long SizeWindow, long SizeSearch, long Weights) const;
  virtual HImage OpticalFlowMatch(const HImageArray &Image2, const HTuple &Mode, const HTuple &Threshold, const HTuple &Step, const HTuple &SizeWindow, const HTuple &SizeSearch, const HTuple &Weights) const;
  virtual HImage OpticalFlowMatch(const HImageArray &Image2, const char *Mode, long Threshold, long Step, long SizeWindow, long SizeSearch, long Weights) const;
  // Matching a template and an image in a resolution pyramid.
  virtual HImage ExhaustiveMatchMg(const HImage &ImageTemplate, const HTuple &Mode, const HTuple &Level, const HTuple &Threshold) const;
  virtual HImage ExhaustiveMatchMg(const HImage &ImageTemplate, const char *Mode, long Level, long Threshold) const;
  // Preparing a pattern for template matching with rotation.
  virtual long CreateTemplateRot(const HTuple &NumLevel, const HTuple &AngleStart, const HTuple &AngleExtend, const HTuple &AngleStep, const HTuple &Optimize, const HTuple &GrayValues) const;
  virtual long CreateTemplateRot(long NumLevel, double AngleStart, double AngleExtend, double AngleStep, const char *Optimize, const char *GrayValues) const;
  // Preparing a pattern for template matching.
  virtual long CreateTemplate(const HTuple &FirstError, const HTuple &NumLevel, const HTuple &Optimize, const HTuple &GrayValues) const;
  virtual long CreateTemplate(long FirstError, long NumLevel, const char *Optimize, const char *GrayValues) const;
  // Adapting a template to the size of an image.
  virtual void AdaptTemplate(const HTuple &TemplateID) const;
  virtual void AdaptTemplate(long TemplateID) const;
  // Searching all good grayvalue matches in a pyramid.
  virtual HRegion FastMatchMg(const HTuple &TemplateID, const HTuple &MaxError, const HTuple &NumLevel) const;
  virtual HRegion FastMatchMg(long TemplateID, double MaxError, long NumLevel) const;
  // Searching the best grayvalue matches in a pyramid.
  virtual double BestMatchMg(const HTuple &TemplateID, const HTuple &MaxError, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &WhichLevels, double *Column, double *Error) const;
  virtual double BestMatchMg(long TemplateID, double MaxError, const char *SubPixel, long NumLevels, long WhichLevels, double *Column, double *Error) const;
  // Searching all good matches of a template and an image.
  virtual HRegion FastMatch(const HTuple &TemplateID, const HTuple &MaxError) const;
  virtual HRegion FastMatch(long TemplateID, double MaxError) const;
  // Searching the best matching of a template and a pyramid with rotation.
  virtual double BestMatchRotMg(const HTuple &TemplateID, const HTuple &AngleStart, const HTuple &AngleExtend, const HTuple &MaxError, const HTuple &SubPixel, const HTuple &NumLevels, double *Column, double *Angle, double *Error) const;
  virtual double BestMatchRotMg(long TemplateID, double AngleStart, double AngleExtend, double MaxError, const char *SubPixel, long NumLevels, double *Column, double *Angle, double *Error) const;
  // Searching the best matching of a template and an image with rotation.
  virtual double BestMatchRot(const HTuple &TemplateID, const HTuple &AngleStart, const HTuple &AngleExtend, const HTuple &MaxError, const HTuple &SubPixel, double *Column, double *Angle, double *Error) const;
  virtual double BestMatchRot(long TemplateID, double AngleStart, double AngleExtend, double MaxError, const char *SubPixel, double *Column, double *Angle, double *Error) const;
  // Searching the best matching of a template and an image.
  virtual double BestMatch(const HTuple &TemplateID, const HTuple &MaxError, const HTuple &SubPixel, double *Column, double *Error) const;
  virtual double BestMatch(long TemplateID, double MaxError, const char *SubPixel, double *Column, double *Error) const;
  // Matching of a template and an image.
  virtual HImage ExhaustiveMatch(const HRegion &RegionOfInterest, const HImage &ImageTemplate, const HTuple &Mode) const;
  virtual HImage ExhaustiveMatch(const HRegion &RegionOfInterest, const HImage &ImageTemplate, const char *Mode) const;
  // Searching corners in images.
  virtual HImage CornerResponse(const HTuple &Size, const HTuple &Weight) const;
  virtual HImage CornerResponse(long Size, double Weight) const;
  // Calculating a Gauss pyramid.
  virtual HImageArray GenGaussPyramid(const HTuple &Mode, const HTuple &Scale) const;
  virtual HImageArray GenGaussPyramid(const char *Mode, double Scale) const;
  // Calculating the monotony operation.
  virtual HImage Monotony(void) const;
  // Expansion of a displacement vector field onto undefined areas.
  virtual HImage FillDvf(const HTuple &Width, const HTuple &Height, const HTuple &Iterations) const;
  virtual HImage FillDvf(long Width, long Height, long Iterations) const;
  // Add noise to an image.
  virtual HImage AddNoiseWhite(const HTuple &Amp) const;
  virtual HImage AddNoiseWhite(double Amp) const;
  // Add noise to an image.
  virtual HImage AddNoiseDistribution(const HTuple &Distribution) const;
  virtual HImage AddNoiseDistribution(double Distribution) const;
  // Find corners using the Sojka operator.
  virtual HTuple PointsSojka(const HTuple &MaskSize, const HTuple &SigmaW, const HTuple &SigmaD, const HTuple &MinGrad, const HTuple &MinApparentness, const HTuple &MinAngle, const HTuple &Subpix, HTuple *Col) const;
  virtual HTuple PointsSojka(long MaskSize, double SigmaW, double SigmaD, double MinGrad, double MinApparentness, double MinAngle, const char *Subpix, HTuple *Col) const;
  // Enhance circular dots in an image.
  virtual HImage DotsImage(const HTuple &Diameter, const HTuple &FilterType, const HTuple &PixelShift) const;
  virtual HImage DotsImage(long Diameter, const char *FilterType, long PixelShift) const;
  // Subpixel precise detection of local minima in an image.
  virtual HTuple LocalMinSubPix(const HTuple &Filter, const HTuple &Sigma, const HTuple &Threshold, HTuple *Col) const;
  virtual HTuple LocalMinSubPix(const char *Filter, double Sigma, double Threshold, HTuple *Col) const;
  // Subpixel precise detection of local maxima in an image.
  virtual HTuple LocalMaxSubPix(const HTuple &Filter, const HTuple &Sigma, const HTuple &Threshold, HTuple *Col) const;
  virtual HTuple LocalMaxSubPix(const char *Filter, double Sigma, double Threshold, HTuple *Col) const;
  // Subpixel precise detection of saddle points in an image.
  virtual HTuple SaddlePointsSubPix(const HTuple &Filter, const HTuple &Sigma, const HTuple &Threshold, HTuple *Col) const;
  virtual HTuple SaddlePointsSubPix(const char *Filter, double Sigma, double Threshold, HTuple *Col) const;
  // Subpixel precise detection of critical points in an image.
  virtual HTuple CriticalPointsSubPix(const HTuple &Filter, const HTuple &Sigma, const HTuple &Threshold, HTuple *ColMin, HTuple *RowMax, HTuple *ColMax, HTuple *RowSaddle, HTuple *ColSaddle) const;
  virtual HTuple CriticalPointsSubPix(const char *Filter, double Sigma, double Threshold, HTuple *ColMin, HTuple *RowMax, HTuple *ColMax, HTuple *RowSaddle, HTuple *ColSaddle) const;
  // Detect points of interest using the Harris operator.
  virtual HTuple PointsHarris(const HTuple &SigmaGrad, const HTuple &SigmaSmooth, const HTuple &Alpha, const HTuple &Threshold, HTuple *Col) const;
  virtual HTuple PointsHarris(double SigmaGrad, double SigmaSmooth, double Alpha, double Threshold, HTuple *Col) const;
  // Detect points of interest using the Frstner operator.
  virtual HTuple PointsFoerstner(const HTuple &SigmaGrad, const HTuple &SigmaInt, const HTuple &SigmaPoints, const HTuple &ThreshInhom, const HTuple &ThreshShape, const HTuple &Smoothing, const HTuple &EliminateDoublets, HTuple *ColJunctions, HTuple *CoRRJunctions, HTuple *CoRCJunctions, HTuple *CoCCJunctions, HTuple *RowArea, HTuple *ColArea, HTuple *CoRRArea, HTuple *CoRCArea, HTuple *CoCCArea) const;
  virtual HTuple PointsFoerstner(double SigmaGrad, double SigmaInt, double SigmaPoints, double ThreshInhom, double ThreshShape, const char *Smoothing, const char *EliminateDoublets, HTuple *ColJunctions, HTuple *CoRRJunctions, HTuple *CoRCJunctions, HTuple *CoCCJunctions, HTuple *RowArea, HTuple *ColArea, HTuple *CoRRArea, HTuple *CoRCArea, HTuple *CoCCArea) const;
  // Expand the domain of an image and set the gray values in the
  // expanded domain.
  virtual HImage ExpandDomainGray(const HTuple &ExpansionRange) const;
  virtual HImage ExpandDomainGray(long ExpansionRange) const;
  // Compute the topographic primal sketch of an image.
  virtual HImage TopographicSketch(void) const;
  // Compute the principal components of multi-channel images.
  virtual HImage PrincipalComp(HTuple *InfoPerComp) const;
  // Perform a grayvalue closing with a selected mask.
  virtual HImage GrayClosingShape(const HTuple &MaskHeight, const HTuple &MaskWidth, const HTuple &MaskShape) const;
  virtual HImage GrayClosingShape(long MaskHeight, long MaskWidth, const char *MaskShape) const;
  // Perform a gray value opening with a selected mask.
  virtual HImage GrayOpeningShape(const HTuple &MaskHeight, const HTuple &MaskWidth, const HTuple &MaskShape) const;
  virtual HImage GrayOpeningShape(long MaskHeight, long MaskWidth, const char *MaskShape) const;
  // Determine the minimum gray value within a selected mask.
  virtual HImage GrayErosionShape(const HTuple &MaskHeight, const HTuple &MaskWidth, const HTuple &MaskShape) const;
  virtual HImage GrayErosionShape(long MaskHeight, long MaskWidth, const char *MaskShape) const;
  // Determine the maximum gray value within a selected mask.
  virtual HImage GrayDilationShape(const HTuple &MaskHeight, const HTuple &MaskWidth, const HTuple &MaskShape) const;
  virtual HImage GrayDilationShape(long MaskHeight, long MaskWidth, const char *MaskShape) const;
  // Determine the gray value range within a rectangle.
  virtual HImage GrayRangeRect(const HTuple &MaskHeight, const HTuple &MaskWidth) const;
  virtual HImage GrayRangeRect(long MaskHeight, long MaskWidth) const;
  // Determine the minimum gray value within a rectangle.
  virtual HImage GrayErosionRect(const HTuple &MaskHeight, const HTuple &MaskWidth) const;
  virtual HImage GrayErosionRect(long MaskHeight, long MaskWidth) const;
  // Determine the maximum gray value within a rectangle.
  virtual HImage GrayDilationRect(const HTuple &MaskHeight, const HTuple &MaskWidth) const;
  virtual HImage GrayDilationRect(long MaskHeight, long MaskWidth) const;
  // Thinning of gray value images.
  virtual HImage GraySkeleton(void) const;
  // Transform an image with a gray-value look-up-table
  virtual HImage LutTrans(const HTuple &Lut) const;
  virtual HImage LutTrans(long Lut) const;
  // Convolve an image with an arbitrary filter mask.
  virtual HImage ConvolImage(const HTuple &FilterMask, const HTuple &Margin) const;
  // Convert the type of an image.
  virtual HImage ConvertImageType(const HTuple &NewType) const;
  virtual HImage ConvertImageType(const char *NewType) const;
  // Convert two int1-images into a displacement vector field.
  virtual HImage Int1ToDvf(const HImage &Col) const;
  virtual HImage Int1ToDvf(const HImageArray &Col) const;
  // Convert a displacement vector field into two int1-images.
  virtual HImage DvfToInt1(HImage *Col) const;
  // Convert two real images into a complex image.
  virtual HImage RealToComplex(const HImage &ImageImaginary) const;
  virtual HImage RealToComplex(const HImageArray &ImageImaginary) const;
  // Convert a complex image into two real images.
  virtual HImage ComplexToReal(HImage *ImageImaginary) const;
  // Calculate the lowest possible gray value on an arbitrary path to the
  // image border for each point in the image.
  virtual HImage GrayInside(void) const;
  // Symmentry of gray values along a row.
  virtual HImage Symmetry(const HTuple &MaskSize, const HTuple &Direction, const HTuple &Exponent) const;
  virtual HImage Symmetry(long MaskSize, double Direction, double Exponent) const;
  // Selection of gray values of a multi channel image using an index image.
  virtual HImage SelectGrayvaluesFromChannels(const HImage &IndexImage) const;
  virtual HImage SelectGrayvaluesFromChannels(const HImageArray &IndexImage) const;
  // Extract depth using mutiple focus levels.
  virtual HImage DepthFromFocus(HImage *Confidence, const HTuple &Filter, const HTuple &Selection) const;
  virtual HImage DepthFromFocus(HImage *Confidence, const char *Filter, const char *Selection) const;
  // Filter an image using a Laws texture filter.
  virtual HImage TextureLaws(const HTuple &FilterTypes, const HTuple &Shift, const HTuple &FilterSize) const;
  virtual HImage TextureLaws(const char *FilterTypes, long Shift, long FilterSize) const;
  // Calculate the standard deviation of gray values within rectangular windows.
  virtual HImage DeviationImage(const HTuple &Width, const HTuple &Height) const;
  virtual HImage DeviationImage(long Width, long Height) const;
  // Calculate the entropy of gray values within a rectangular window.
  virtual HImage EntropyImage(const HTuple &Width, const HTuple &Height) const;
  virtual HImage EntropyImage(long Width, long Height) const;
  // Histogram linearisation of images
  virtual HImage EquHistoImage(void) const;
  // Illuminate image.
  virtual HImage Illuminate(const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &Factor) const;
  virtual HImage Illuminate(long MaskWidth, long MaskHeight, double Factor) const;
  // Enhance contrast of the image.
  virtual HImage Emphasize(const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &Factor) const;
  virtual HImage Emphasize(long MaskWidth, long MaskHeight, double Factor) const;
  // Maximum gray value spreading in the value range 
  // 0 bis 255.
  virtual HImage ScaleImageMax(void) const;
  // Displays gray value images.
  virtual void DispImage(const HTuple &WindowHandle) const;
  virtual void DispImage(long WindowHandle) const;
  // Displays images with several channels.
  virtual void DispChannel(const HTuple &WindowHandle, const HTuple &Channel) const;
  virtual void DispChannel(long WindowHandle, long Channel) const;
  // Displays a color (RGB) image
  virtual void DispColor(const HTuple &WindowHandle) const;
  virtual void DispColor(long WindowHandle) const;
  // Visualize images using gnuplot.
  virtual void GnuplotPlotImage(const HTuple &GnuplotFileID, const HTuple &SamplesX, const HTuple &SamplesY, const HTuple &ViewRotX, const HTuple &ViewRotZ, const HTuple &Hidden3D) const;
  virtual void GnuplotPlotImage(long GnuplotFileID, long SamplesX, long SamplesY, double ViewRotX, double ViewRotZ, const char *Hidden3D) const;
  // Segment the rectification grid region in the image.
  virtual HRegion FindRectificationGrid(const HTuple &MinContrast, const HTuple &Radius) const;
  virtual HRegion FindRectificationGrid(double MinContrast, double Radius) const;
  // Establish connections between the grid points of the rectification grid.
  virtual HXLDArray ConnectGridPoints(const HTuple &Row, const HTuple &Col, const HTuple &Sigma, const HTuple &MaxDist) const;
  virtual HXLDArray ConnectGridPoints(double Row, double Col, long Sigma, double MaxDist) const;
  // Compute the mapping between the distorted image and the rectified image
  // based upon the points of a regular grid.
  virtual HImage GenGridRectificationMap(const HXLDArray &ConnectingLines, HXLDArray *Meshes, const HTuple &GridSpacing, const HTuple &Rotation, const HTuple &Row, const HTuple &Col) const;
  // Detect lines in edge images with the help of the Hough transform 
  // using local gradient direction and return them in normal form.
  virtual HImage HoughLinesDir(HRegionArray *Lines, const HTuple &DirectionUncertainty, const HTuple &AngleResolution, const HTuple &Smoothing, const HTuple &FilterSize, const HTuple &Threshold, const HTuple &AngleGap, const HTuple &DistGap, const HTuple &GenLines, HTuple *Angle, HTuple *Dist) const;
  virtual HImage HoughLinesDir(HRegionArray *Lines, long DirectionUncertainty, long AngleResolution, const char *Smoothing, long FilterSize, long Threshold, long AngleGap, long DistGap, const char *GenLines, HTuple *Angle, HTuple *Dist) const;
  // Compute the Hough transform for lines using local gradient direction.
  virtual HImage HoughLineTransDir(const HTuple &DirectionUncertainty, const HTuple &AngleResolution) const;
  virtual HImage HoughLineTransDir(long DirectionUncertainty, long AngleResolution) const;
  // Apply a general transformation to an image.
  virtual HImage MapImage(const HImage &Map) const;
  // Generate a projection map that describes the mapping of images corresponding
  // to a changing radial distortion.
  static HImage GenRadialDistortionMap(const HTuple &CamParIn, const HTuple &CamParOut, const HTuple &Interpolation);
  static HImage GenRadialDistortionMap(double CamParIn, double CamParOut, const char *Interpolation);
  // Generate a projection map that describes the mapping between the image 
  // plane and a world plane.
  static HImage GenImageToWorldPlaneMap(const HTuple &CamParam, const HTuple &CamPose, const HTuple &WidthIn, const HTuple &HeightIn, const HTuple &WidthMapped, const HTuple &HeightMapped, const HTuple &Scale, const HTuple &Interpolation);
  // Rectify an image to a world plane represented by external
  // camera parameters.
  virtual HImage ImageToWorldPlane(const HTuple &CamParam, const HTuple &CamPose, const HTuple &Width, const HTuple &Height, const HTuple &Scale, const HTuple &Interpolation) const;
  // Change the radial distortion of an image.
  virtual HImage ChangeRadialDistortionImage(const HRegion &Region, const HTuple &CamParIn, const HTuple &CamParOut) const;
  virtual HImage ChangeRadialDistortionImage(const HRegion &Region, double CamParIn, double CamParOut) const;
  // Simulate a video image with calibration table.
  static HImage SimCaltab(const HTuple &CalTabDescrFile, const HTuple &CamParam, const HTuple &CamPose, const HTuple &GrayBackground, const HTuple &GrayCaltab, const HTuple &GrayMarks, const HTuple &ScaleFac);
  static HImage SimCaltab(const char *CalTabDescrFile, double CamParam, double CamPose, long GrayBackground, long GrayCaltab, long GrayMarks, double ScaleFac);
  // Extract the 2D calibration marks from the video image and 
  // calculate initial values for the external camera parameters.
  virtual HTuple FindMarksAndPose(const HRegion &CalTabRegion, const HTuple &CalTabDescrFile, const HTuple &StartCamParam, const HTuple &StartThresh, const HTuple &DeltaThresh, const HTuple &MinThresh, const HTuple &Alpha, const HTuple &MinContLength, const HTuple &MaxDiamMarks, HTuple *CCoord, HTuple *StartPose) const;
  virtual HTuple FindMarksAndPose(const HRegion &CalTabRegion, const char *CalTabDescrFile, double StartCamParam, long StartThresh, long DeltaThresh, long MinThresh, double Alpha, double MinContLength, double MaxDiamMarks, HTuple *CCoord, HTuple *StartPose) const;
  // Segment the calibration table region in the image.
  virtual HRegion FindCaltab(const HTuple &CalTabDescrFile, const HTuple &SizeGauss, const HTuple &MarkThresh, const HTuple &MinDiamMarks) const;
  virtual HRegion FindCaltab(const char *CalTabDescrFile, long SizeGauss, long MarkThresh, long MinDiamMarks) const;
  // Adopt new parameters that are used to create the model components into the 
  // training result.
  virtual HRegion ClusterModelComponents(const HTuple &ComponentTrainingID, const HTuple &AmbiguityCriterion, const HTuple &MaxContourOverlap, const HTuple &ClusterThreshold) const;
  virtual HRegion ClusterModelComponents(long ComponentTrainingID, const char *AmbiguityCriterion, double MaxContourOverlap, double ClusterThreshold) const;
  // Train components and relations for the component-based matching.
  virtual HRegion TrainModelComponents(const HRegionArray &InitialComponents, const HImage &TrainingImages, const HTuple &ContrastLow, const HTuple &ContrastHigh, const HTuple &MinSize, const HTuple &MinScore, const HTuple &SearchRowTol, const HTuple &SearchColumnTol, const HTuple &SearchAngleTol, const HTuple &TrainingEmphasis, const HTuple &AmbiguityCriterion, const HTuple &MaxContourOverlap, const HTuple &ClusterThreshold, long *ComponentTrainingID) const;
  virtual HRegion TrainModelComponents(const HRegionArray &InitialComponents, const HImage &TrainingImages, long ContrastLow, long ContrastHigh, long MinSize, double MinScore, long SearchRowTol, long SearchColumnTol, double SearchAngleTol, const char *TrainingEmphasis, const char *AmbiguityCriterion, double MaxContourOverlap, double ClusterThreshold, long *ComponentTrainingID) const;
  virtual HRegionArray TrainModelComponents(const HRegionArray &InitialComponents, const HImageArray &TrainingImages, const HTuple &ContrastLow, const HTuple &ContrastHigh, const HTuple &MinSize, const HTuple &MinScore, const HTuple &SearchRowTol, const HTuple &SearchColumnTol, const HTuple &SearchAngleTol, const HTuple &TrainingEmphasis, const HTuple &AmbiguityCriterion, const HTuple &MaxContourOverlap, const HTuple &ClusterThreshold, HTuple *ComponentTrainingID) const;
  virtual HRegionArray TrainModelComponents(const HRegionArray &InitialComponents, const HImageArray &TrainingImages, long ContrastLow, long ContrastHigh, long MinSize, double MinScore, long SearchRowTol, long SearchColumnTol, double SearchAngleTol, const char *TrainingEmphasis, const char *AmbiguityCriterion, double MaxContourOverlap, double ClusterThreshold, HTuple *ComponentTrainingID) const;
  // Extract the initial components of a component model.
  virtual HRegionArray GenInitialComponents(const HTuple &ContrastLow, const HTuple &ContrastHigh, const HTuple &MinSize, const HTuple &Mode, const HTuple &GenericName, const HTuple &GenericValue) const;
  virtual HRegionArray GenInitialComponents(long ContrastLow, long ContrastHigh, long MinSize, const char *Mode, const char *GenericName, double GenericValue) const;
  // Create a curved gray surface with second order polynomial.
  static HImage GenImageSurfaceSecondOrder(const HTuple &Type, const HTuple &Alpha, const HTuple &Beta, const HTuple &Gamma, const HTuple &Delta, const HTuple &Epsilon, const HTuple &Zeta, const HTuple &Row, const HTuple &Col, const HTuple &Width, const HTuple &Height);
  static HImage GenImageSurfaceSecondOrder(const char *Type, double Alpha, double Beta, double Gamma, double Delta, double Epsilon, double Zeta, double Row, double Col, long Width, long Height);
  // Create a curved gray surface with first order polynomial.
  static HImage GenImageSurfaceFirstOrder(const HTuple &Type, const HTuple &Alpha, const HTuple &Beta, const HTuple &Gamma, const HTuple &Row, const HTuple &Col, const HTuple &Width, const HTuple &Height);
  static HImage GenImageSurfaceFirstOrder(const char *Type, double Alpha, double Beta, double Gamma, double Row, double Col, long Width, long Height);
  // Calculate gray value features from a co-occurrence matrix.
  virtual double CoocFeatureMatrix(HTuple *Correlation, HTuple *Homogeneity, HTuple *Contrast) const;
  // Perform a gray value bottom hat transformation on an image.
  virtual HImage GrayBothat(const HImage &SE) const;
  // Perform a gray value top hat transformation on an image.
  virtual HImage GrayTophat(const HImage &SE) const;
  // Perform a gray value closing on an image.
  virtual HImage GrayClosing(const HImage &SE) const;
  // Perform a gray value opening on an image.
  virtual HImage GrayOpening(const HImage &SE) const;
  // Perform a gray value dilation on an image.
  virtual HImage GrayDilation(const HImage &SE) const;
  // Perform a gray value erosion on an image.
  virtual HImage GrayErosion(const HImage &SE) const;
  // Load a structuring element for gray morphology.
  static HImage ReadGraySe(const HTuple &FileName);
  static HImage ReadGraySe(const char *FileName);
  // Generate ellipsoidal structuring elements for gray morphology.
  static HImage GenDiscSe(const HTuple &Width, const HTuple &Height, const HTuple &Smax);
  static HImage GenDiscSe(long Width, long Height, long Smax);
  // Cut out an image area relative to the domain.
  virtual HImage CropDomainRel(const HTuple &Top, const HTuple &Left, const HTuple &Bottom, const HTuple &Right) const;
  virtual HImage CropDomainRel(long Top, long Left, long Bottom, long Right) const;
  // Access the features which correspond to a character.
  virtual HTuple OcrGetFeatures(const HTuple &OcrHandle) const;
  virtual HTuple OcrGetFeatures(long OcrHandle) const;
  // Write characters into a training file.
  virtual void WriteOcrTrainfImage(const HTuple &Class, const HTuple &FileName) const;
  virtual void WriteOcrTrainfImage(const char *Class, const char *FileName) const;
  // Verification of a pattern using an OCV tool.
  virtual double DoOcvSimple(const HTuple &OCVHandle, const HTuple &PatternName, const HTuple &AdaptPos, const HTuple &AdaptSize, const HTuple &AdaptAngle, const HTuple &AdaptGray, const HTuple &Threshold) const;
  virtual double DoOcvSimple(long OCVHandle, const char *PatternName, const char *AdaptPos, const char *AdaptSize, const char *AdaptAngle, const char *AdaptGray, double Threshold) const;
  // Training of an OCV tool.
  virtual void TraindOcvProj(const HTuple &OCVHandle, const HTuple &Name, const HTuple &Mode) const;
  virtual void TraindOcvProj(long OCVHandle, const char *Name, const char *Mode) const;
  // Create an image from three pointers to the pixels (red/green/blue).
  static HImage GenImage3(const HTuple &Type, const HTuple &Width, const HTuple &Height, const HTuple &PixelPointerRed, const HTuple &PixelPointerGreen, const HTuple &PixelPointerBlue);
  static HImage GenImage3(const char *Type, long Width, long Height, long PixelPointerRed, long PixelPointerGreen, long PixelPointerBlue);
  // Create an image from a pointer to the pixels.
  static HImage GenImage1(const HTuple &Type, const HTuple &Width, const HTuple &Height, const HTuple &PixelPointer);
  static HImage GenImage1(const char *Type, long Width, long Height, long PixelPointer);
  // Create an image with constant gray value.
  static HImage GenImageConst(const HTuple &Type, const HTuple &Width, const HTuple &Height);
  static HImage GenImageConst(const char *Type, long Width, long Height);
  // Create a gray value ramp.
  static HImage GenImageGrayRamp(const HTuple &Alpha, const HTuple &Beta, const HTuple &Mean, const HTuple &Row, const HTuple &Column, const HTuple &Width, const HTuple &Height);
  static HImage GenImageGrayRamp(double Alpha, double Beta, double Mean, long Row, long Column, long Width, long Height);
  // Create an image from a pointer on the pixels with storage management.
  static HImage GenImage1Extern(const HTuple &Type, const HTuple &Width, const HTuple &Height, const HTuple &PixelPointer, const HTuple &ClearProc);
  static HImage GenImage1Extern(const char *Type, long Width, long Height, long PixelPointer, long ClearProc);
  // Create an image with a rectangular domain from a pointer on the pixels
  // (with storage management).
  static HImage GenImage1Rect(const HTuple &PixelPointer, const HTuple &Width, const HTuple &Height, const HTuple &VerticalPitch, const HTuple &HorizontalBitPitch, const HTuple &BitsPerPixel, const HTuple &DoCopy, const HTuple &ClearProc);
  static HImage GenImage1Rect(long PixelPointer, long Width, long Height, long VerticalPitch, long HorizontalBitPitch, long BitsPerPixel, const char *DoCopy, long ClearProc);
  // Set single gray values in an image.
  virtual void SetGrayval(const HTuple &Row, const HTuple &Column, const HTuple &Grayval) const;
  virtual void SetGrayval(long Row, long Column, double Grayval) const;
  // Paint a region in the image with a constant gray value.
  virtual void OverpaintRegion(const HRegion &Region, const HTuple &Grayval, const HTuple &Type) const;
  virtual void OverpaintRegion(const HRegion &Region, double Grayval, const char *Type) const;
  virtual void OverpaintRegion(const HRegionArray &Region, const HTuple &Grayval, const HTuple &Type) const;
  virtual void OverpaintRegion(const HRegionArray &Region, double Grayval, const char *Type) const;
  // Set the gray values of an image to a specified value.
  virtual HImage GenImageProto(const HTuple &Grayval) const;
  virtual HImage GenImageProto(double Grayval) const;
  // Copy the gray values of an image into another image.
  virtual HImage PaintGray(const HImage &ImageDestination) const;
  // Overwrites the gray values of an image.
  virtual void OverpaintGray(const HImage &ImageSource) const;
  // Copy an image and allocate new memory for it.
  virtual HImage CopyImage(void) const;
  // Access to the image data pointer and the image data inside 
  // the smallest rectangle of the domain of the input image.
  virtual long GetImagePointer1Rect(HTuple *Width, HTuple *Height, HTuple *VerticalPitch, HTuple *HorizontalBitPitch, HTuple *BitsPerPixel) const;
  // Access the pointers of a colored image.
  virtual long GetImagePointer3(HTuple *PointerGreen, HTuple *PointerBlue, HTuple *Type, HTuple *Width, HTuple *Height) const;
  // Access the pointer of a channel.
  virtual long GetImagePointer1(HTuple *Type, HTuple *Width, HTuple *Height) const;
  // Request time at which the image was created.
  virtual long GetImageTime(HTuple *Second, HTuple *Minute, HTuple *Hour, HTuple *Day, HTuple *YDay, HTuple *Month, HTuple *Year) const;
  // Access the gray values of an image object.
  virtual HTuple GetGrayval(const HTuple &Row, const HTuple &Column) const;
  virtual HTuple GetGrayval(long Row, long Column) const;
  // Classify pixels using hyper-cuboids.
  virtual HRegionArray ClassNdimBox(const HTuple &ClassifHandle) const;
  virtual HRegionArray ClassNdimBox(long ClassifHandle) const;
  // Classify pixels using hyper-spheres or hyper-cubes.
  virtual HRegionArray ClassNdimNorm(const HTuple &Metric, const HTuple &SingleMultiple, const HTuple &Radius, const HTuple &Center) const;
  virtual HRegionArray ClassNdimNorm(const char *Metric, const char *SingleMultiple, double Radius, double Center) const;
  // Segment an image using two-dimensional pixel classification.
  virtual HRegion Class2dimSup(const HImage &ImageRow, const HRegion &FeatureSpace) const;
  virtual HRegion Class2dimSup(const HImageArray &ImageRow, const HRegionArray &FeatureSpace) const;
  // Segment two images by clustering.
  virtual HRegionArray Class2dimUnsup(const HImage &Image2, const HTuple &Threshold, const HTuple &NumClasses) const;
  virtual HRegionArray Class2dimUnsup(const HImage &Image2, long Threshold, long NumClasses) const;
  // Compare two images pixel by pixel.
  virtual HRegion CheckDifference(const HImage &Pattern, const HTuple &Mode, const HTuple &DiffLowerBound, const HTuple &DiffUpperBound, const HTuple &GrayOffset, const HTuple &AddRow, const HTuple &AddCol) const;
  virtual HRegion CheckDifference(const HImage &Pattern, const char *Mode, long DiffLowerBound, long DiffUpperBound, long GrayOffset, long AddRow, long AddCol) const;
  virtual HRegion CheckDifference(const HImageArray &Pattern, const HTuple &Mode, const HTuple &DiffLowerBound, const HTuple &DiffUpperBound, const HTuple &GrayOffset, const HTuple &AddRow, const HTuple &AddCol) const;
  virtual HRegion CheckDifference(const HImageArray &Pattern, const char *Mode, long DiffLowerBound, long DiffUpperBound, long GrayOffset, long AddRow, long AddCol) const;
  // Perform a threshold segmentation for extracting characters.
  virtual HRegion CharThreshold(const HRegion &HistoRegion, const HTuple &Sigma, const HTuple &Percent, long *Threshold) const;
  virtual HRegion CharThreshold(const HRegion &HistoRegion, double Sigma, double Percent, long *Threshold) const;
  // Extract regions with equal gray values from an image.
  virtual HRegionArray LabelToRegion(void) const;
  // Suppress non-maximum points on an edge.
  virtual HImage NonmaxSuppressionAmp(const HTuple &Mode) const;
  virtual HImage NonmaxSuppressionAmp(const char *Mode) const;
  // Suppress non-maximum points on an edge.
  virtual HImage NonmaxSuppressionDir(const HImage &ImgDir, const HTuple &Mode) const;
  virtual HImage NonmaxSuppressionDir(const HImage &ImgDir, const char *Mode) const;
  virtual HImage NonmaxSuppressionDir(const HImageArray &ImgDir, const HTuple &Mode) const;
  virtual HImage NonmaxSuppressionDir(const HImageArray &ImgDir, const char *Mode) const;
  // Perform a hysteresis threshold operation on an image.
  virtual HRegion HysteresisThreshold(const HTuple &Low, const HTuple &High, const HTuple &MaxLength) const;
  virtual HRegion HysteresisThreshold(long Low, long High, long MaxLength) const;
  // Segment an image using a local threshold.
  virtual HRegion DynThreshold(const HImage &ThresholdImage, const HTuple &Offset, const HTuple &LightDark) const;
  virtual HRegion DynThreshold(const HImage &ThresholdImage, double Offset, const char *LightDark) const;
  virtual HRegion DynThreshold(const HImageArray &ThresholdImage, const HTuple &Offset, const HTuple &LightDark) const;
  virtual HRegion DynThreshold(const HImageArray &ThresholdImage, double Offset, const char *LightDark) const;
  // Select gray values lying within an interval.
  virtual HRegion Threshold(const HTuple &MinGray, const HTuple &MaxGray) const;
  virtual HRegion Threshold(double MinGray, double MaxGray) const;
  // Extract level crossings from an image with subpixel accuracy.
  virtual HXLDContArray ThresholdSubPix(const HTuple &Threshold) const;
  virtual HXLDContArray ThresholdSubPix(double Threshold) const;
  // Regiongrowing for multi-channel images.
  virtual HRegionArray RegiongrowingN(const HTuple &Metric, const HTuple &MinTolerance, const HTuple &MaxTolerance, const HTuple &MinSize) const;
  virtual HRegionArray RegiongrowingN(const char *Metric, double MinTolerance, double MaxTolerance, long MinSize) const;
  // Segment an image using regiongrowing.
  virtual HRegionArray Regiongrowing(const HTuple &Row, const HTuple &Column, const HTuple &Tolerance, const HTuple &MinSize) const;
  virtual HRegionArray Regiongrowing(long Row, long Column, double Tolerance, long MinSize) const;
  // Regiongrowing using mean gray values.
  virtual HRegionArray RegiongrowingMean(const HTuple &StartRows, const HTuple &StartColumns, const HTuple &Tolerance, const HTuple &MinSize) const;
  virtual HRegionArray RegiongrowingMean(long StartRows, long StartColumns, double Tolerance, long MinSize) const;
  // Segment an image by ``pouring water'' over it.
  virtual HRegionArray Pouring(const HTuple &Mode, const HTuple &MinGray, const HTuple &MaxGray) const;
  virtual HRegionArray Pouring(const char *Mode, long MinGray, long MaxGray) const;
  // Extract watersheds and basins from an image.
  virtual HRegionArray Watersheds(HRegion *Watersheds) const;
  // Detect zero crossings in an image.
  virtual HRegion ZeroCrossing(void) const;
  // Extract zero crossings from an image with subpixel accuracy.
  virtual HXLDContArray ZeroCrossingSubPix(void) const;
  // Threshold operator for signed images.
  virtual HRegionArray DualThreshold(const HTuple &MinSize, const HTuple &MinGray, const HTuple &Threshold) const;
  virtual HRegionArray DualThreshold(long MinSize, double MinGray, double Threshold) const;
  // Expand a region starting at a given line.
  virtual HRegion ExpandLine(const HTuple &Index, const HTuple &ExpandType, const HTuple &RowColumn, const HTuple &Threshold) const;
  virtual HRegion ExpandLine(long Index, const char *ExpandType, const char *RowColumn, double Threshold) const;
  // Detect all local minima in an image.
  virtual HRegion LocalMin(void) const;
  // Detect all gray value lowlands.
  virtual HRegionArray Lowlands(void) const;
  // Detect the centers of all gray value lowlands.
  virtual HRegionArray LowlandsCenter(void) const;
  // Detect all local maxima in an image.
  virtual HRegion LocalMax(void) const;
  // Detect all gray value plateaus.
  virtual HRegionArray Plateaus(void) const;
  // Detect the centers of all gray value plateaus.
  virtual HRegionArray PlateausCenter(void) const;
  // Segment an image using thresholds determined from its histogram.
  virtual HRegionArray AutoThreshold(const HTuple &Sigma) const;
  virtual HRegionArray AutoThreshold(double Sigma) const;
  // Segment a black-and-white image using an automatically determined
  // threshold.
  virtual HRegion BinThreshold(void) const;
  // Fast selection of grayvalues within a given gray intervall.
  virtual HRegion FastThreshold(const HTuple &MinGray, const HTuple &MaxGray, const HTuple &MinHeight) const;
  virtual HRegion FastThreshold(double MinGray, double MaxGray, long MinHeight) const;
  // Shade a height field.
  virtual HImage ShadeHeightField(const HTuple &Slant, const HTuple &Tilt, const HTuple &Albedo, const HTuple &Ambient, const HTuple &Shadows) const;
  virtual HImage ShadeHeightField(double Slant, double Tilt, double Albedo, double Ambient, const char *Shadows) const;
  // Estimate the albedo of a surface and the amount of ambient light.
  virtual double EstimateAlAm(double *Ambient) const;
  // Estimate the slant of a light source and the albedo of a surface.
  virtual double EstimateSlAlZc(double *Albedo) const;
  // Estimate the slant of a light source and the albedo of a surface.
  virtual double EstimateSlAlLr(double *Albedo) const;
  // Estimate the tilt of a light source.
  virtual double EstimateTiltZc(void) const;
  // Estimate the tilt of a light source.
  virtual double EstimateTiltLr(void) const;
  // Reconstruct a surface from at least three gray value images.
  virtual HImage PhotStereo(const HTuple &Slants, const HTuple &Tilts) const;
  virtual HImage PhotStereo(double Slants, double Tilts) const;
  // Reconstruct a surface from a gray value image.
  virtual HImage SfsPentland(const HTuple &Slant, const HTuple &Tilt, const HTuple &Albedo, const HTuple &Ambient) const;
  virtual HImage SfsPentland(double Slant, double Tilt, double Albedo, double Ambient) const;
  // Reconstruct a surface from a gray value image.
  virtual HImage SfsOrigLr(const HTuple &Slant, const HTuple &Tilt, const HTuple &Albedo, const HTuple &Ambient) const;
  virtual HImage SfsOrigLr(double Slant, double Tilt, double Albedo, double Ambient) const;
  // Reconstruct a surface from a gray value image.
  virtual HImage SfsModLr(const HTuple &Slant, const HTuple &Tilt, const HTuple &Albedo, const HTuple &Ambient) const;
  virtual HImage SfsModLr(double Slant, double Tilt, double Albedo, double Ambient) const;
  // Compute the distance values for a rectified stereo image pair.
  virtual HImage BinocularDistance(const HImage &Image2, HImage *Score, const HTuple &CamParamRect1, const HTuple &CamParamRect2, const HTuple &RelPoseRect, const HTuple &Method, const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &TextureThresh, const HTuple &MinDisparity, const HTuple &MaxDisparity, const HTuple &NumLevels, const HTuple &ScoreThresh, const HTuple &Filter, const HTuple &SubDistance) const;
  virtual HImage BinocularDistance(const HImage &Image2, HImage *Score, double CamParamRect1, double CamParamRect2, double RelPoseRect, const char *Method, long MaskWidth, long MaskHeight, double TextureThresh, double MinDisparity, double MaxDisparity, long NumLevels, double ScoreThresh, const char *Filter, const char *SubDistance) const;
  // Compute the disparities of a rectified image pair.
  virtual HImage BinocularDisparity(const HImage &Image2, HImage *Score, const HTuple &Method, const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &TextureThresh, const HTuple &MinDisparity, const HTuple &MaxDisparity, const HTuple &NumLevels, const HTuple &ScoreThresh, const HTuple &Filter, const HTuple &SubDisparity) const;
  virtual HImage BinocularDisparity(const HImage &Image2, HImage *Score, const char *Method, long MaskWidth, long MaskHeight, double TextureThresh, long MinDisparity, long MaxDisparity, long NumLevels, double ScoreThresh, const char *Filter, const char *SubDisparity) const;
  // Generate transformation maps that describe the mapping of the images of a 
  // binocular camera pair to a common rectified image plane.
  static HImage GenBinocularRectificationMap(HImage *Map2, const HTuple &CamParam1, const HTuple &CamParam2, const HTuple &RelPose, const HTuple &SubSampling, const HTuple &Method, const HTuple &Interpolation, HTuple *CamParamRect1, HTuple *CamParamRect2, HTuple *CamPoseRect1, HTuple *CamPoseRect2, HTuple *RelPoseRect);
  static HImage GenBinocularRectificationMap(HImage *Map2, double CamParam1, double CamParam2, double RelPose, double SubSampling, const char *Method, const char *Interpolation, HTuple *CamParamRect1, HTuple *CamParamRect2, HTuple *CamPoseRect1, HTuple *CamPoseRect2, HTuple *RelPoseRect);
  // Receive an image over a socket connection.
  static HImage ReceiveImage(const HTuple &Socket);
  static HImage ReceiveImage(long Socket);
  // Send an image over a socket connection.
  virtual void SendImage(const HTuple &Socket) const;
  virtual void SendImage(long Socket) const;
  // Compute a projective transformation matrix between two images by
  // finding correspondences between points.
  virtual HTuple ProjMatchPointsRansac(const HImage &Image2, const HTuple &Rows1, const HTuple &Cols1, const HTuple &Rows2, const HTuple &Cols2, const HTuple &GrayMatchMethod, const HTuple &MaskSize, const HTuple &RowMove, const HTuple &ColMove, const HTuple &RowTolerance, const HTuple &ColTolerance, const HTuple &Rotation, const HTuple &MatchThreshold, const HTuple &EstimationMethod, const HTuple &DistanceThreshold, const HTuple &RandSeed, HTuple *Points1, HTuple *Points2) const;
  virtual HTuple ProjMatchPointsRansac(const HImage &Image2, double Rows1, double Cols1, double Rows2, double Cols2, const char *GrayMatchMethod, long MaskSize, long RowMove, long ColMove, long RowTolerance, long ColTolerance, double Rotation, long MatchThreshold, const char *EstimationMethod, double DistanceThreshold, long RandSeed, HTuple *Points1, HTuple *Points2) const;
  // Determine the parameters of a shape model.
  virtual HTuple DetermineShapeModelParams(const HTuple &NumLevels, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &ScaleMin, const HTuple &ScaleMax, const HTuple &Optimization, const HTuple &Metric, const HTuple &Contrast, const HTuple &MinContrast, const HTuple &Parameters, HTuple *ParameterValue) const;
  virtual HTuple DetermineShapeModelParams(long NumLevels, double AngleStart, double AngleExtent, double ScaleMin, double ScaleMax, const char *Optimization, const char *Metric, long Contrast, long MinContrast, const char *Parameters, HTuple *ParameterValue) const;
  // Find the best matches of multiple scale invariant shape models.
  virtual HTuple FindScaledShapeModels(const HTuple &ModelIDs, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &ScaleMin, const HTuple &ScaleMax, const HTuple &MinScore, const HTuple &NumMatches, const HTuple &MaxOverlap, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &Greediness, HTuple *Column, HTuple *Angle, HTuple *Scale, HTuple *Score, HTuple *Model) const;
  virtual HTuple FindScaledShapeModels(long ModelIDs, double AngleStart, double AngleExtent, double ScaleMin, double ScaleMax, double MinScore, long NumMatches, double MaxOverlap, const char *SubPixel, long NumLevels, double Greediness, HTuple *Column, HTuple *Angle, HTuple *Scale, HTuple *Score, HTuple *Model) const;
  // Find the best matches of multiple shape models.
  virtual HTuple FindShapeModels(const HTuple &ModelIDs, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &MinScore, const HTuple &NumMatches, const HTuple &MaxOverlap, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &Greediness, HTuple *Column, HTuple *Angle, HTuple *Score, HTuple *Model) const;
  virtual HTuple FindShapeModels(long ModelIDs, double AngleStart, double AngleExtent, double MinScore, long NumMatches, double MaxOverlap, const char *SubPixel, long NumLevels, double Greediness, HTuple *Column, HTuple *Angle, HTuple *Score, HTuple *Model) const;
  // Find the best matches of a scale invariant shape model in an image.
  virtual HTuple FindScaledShapeModel(const HTuple &ModelID, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &ScaleMin, const HTuple &ScaleMax, const HTuple &MinScore, const HTuple &NumMatches, const HTuple &MaxOverlap, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &Greediness, HTuple *Column, HTuple *Angle, HTuple *Scale, HTuple *Score) const;
  virtual HTuple FindScaledShapeModel(long ModelID, double AngleStart, double AngleExtent, double ScaleMin, double ScaleMax, double MinScore, long NumMatches, double MaxOverlap, const char *SubPixel, long NumLevels, double Greediness, HTuple *Column, HTuple *Angle, HTuple *Scale, HTuple *Score) const;
  // Find the best matches of a shape model in an image.
  virtual HTuple FindShapeModel(const HTuple &ModelID, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &MinScore, const HTuple &NumMatches, const HTuple &MaxOverlap, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &Greediness, HTuple *Column, HTuple *Angle, HTuple *Score) const;
  virtual HTuple FindShapeModel(long ModelID, double AngleStart, double AngleExtent, double MinScore, long NumMatches, double MaxOverlap, const char *SubPixel, long NumLevels, double Greediness, HTuple *Column, HTuple *Angle, HTuple *Score) const;
  // Prepare a shape model for scale invariant matching.
  virtual long CreateScaledShapeModel(const HTuple &NumLevels, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &AngleStep, const HTuple &ScaleMin, const HTuple &ScaleMax, const HTuple &ScaleStep, const HTuple &Optimization, const HTuple &Metric, const HTuple &Contrast, const HTuple &MinContrast) const;
  virtual long CreateScaledShapeModel(long NumLevels, double AngleStart, double AngleExtent, double AngleStep, double ScaleMin, double ScaleMax, double ScaleStep, const char *Optimization, const char *Metric, long Contrast, long MinContrast) const;
  // Prepare a shape model for matching.
  virtual long CreateShapeModel(const HTuple &NumLevels, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &AngleStep, const HTuple &Optimization, const HTuple &Metric, const HTuple &Contrast, const HTuple &MinContrast) const;
  virtual long CreateShapeModel(long NumLevels, double AngleStart, double AngleExtent, double AngleStep, const char *Optimization, const char *Metric, long Contrast, long MinContrast) const;
  // Create the representation of a shape model.
  virtual HImageArray InspectShapeModel(HRegionArray *ModelRegions, const HTuple &NumLevels, const HTuple &Contrast) const;
  virtual HImageArray InspectShapeModel(HRegionArray *ModelRegions, long NumLevels, long Contrast) const;
  // Extracting points with a particular grey value along a rectangle or an 
  // annular arc.
  virtual HTuple MeasureThresh(const HTuple &MeasureHandle, const HTuple &Sigma, const HTuple &Threshold, const HTuple &Select, HTuple *ColumnThresh, HTuple *Distance) const;
  virtual HTuple MeasureThresh(long MeasureHandle, double Sigma, double Threshold, const char *Select, HTuple *ColumnThresh, HTuple *Distance) const;
  // Extract a gray value profile perpendicular to a rectangle or annular arc.
  virtual HTuple MeasureProjection(const HTuple &MeasureHandle) const;
  virtual HTuple MeasureProjection(long MeasureHandle) const;
  // Extract straight edge pairs perpendicular to a rectangle or an
  // annular arc.
  virtual HTuple FuzzyMeasurePairing(const HTuple &MeasureHandle, const HTuple &Sigma, const HTuple &AmpThresh, const HTuple &FuzzyThresh, const HTuple &Transition, const HTuple &Pairing, const HTuple &NumPairs, HTuple *ColumnEdgeFirst, HTuple *AmplitudeFirst, HTuple *RowEdgeSecond, HTuple *ColumnEdgeSecond, HTuple *AmplitudeSecond, HTuple *RowPairCenter, HTuple *ColumnPairCenter, HTuple *FuzzyScore, HTuple *IntraDistance) const;
  virtual HTuple FuzzyMeasurePairing(long MeasureHandle, double Sigma, double AmpThresh, double FuzzyThresh, const char *Transition, const char *Pairing, long NumPairs, HTuple *ColumnEdgeFirst, HTuple *AmplitudeFirst, HTuple *RowEdgeSecond, HTuple *ColumnEdgeSecond, HTuple *AmplitudeSecond, HTuple *RowPairCenter, HTuple *ColumnPairCenter, HTuple *FuzzyScore, HTuple *IntraDistance) const;
  // Extract straight edge pairs perpendicular to a rectangle or an annular arc.
  virtual HTuple FuzzyMeasurePairs(const HTuple &MeasureHandle, const HTuple &Sigma, const HTuple &AmpThresh, const HTuple &FuzzyThresh, const HTuple &Transition, HTuple *ColumnEdgeFirst, HTuple *AmplitudeFirst, HTuple *RowEdgeSecond, HTuple *ColumnEdgeSecond, HTuple *AmplitudeSecond, HTuple *RowEdgeCenter, HTuple *ColumnEdgeCenter, HTuple *FuzzyScore, HTuple *IntraDistance, HTuple *InterDistance) const;
  virtual HTuple FuzzyMeasurePairs(long MeasureHandle, double Sigma, double AmpThresh, double FuzzyThresh, const char *Transition, HTuple *ColumnEdgeFirst, HTuple *AmplitudeFirst, HTuple *RowEdgeSecond, HTuple *ColumnEdgeSecond, HTuple *AmplitudeSecond, HTuple *RowEdgeCenter, HTuple *ColumnEdgeCenter, HTuple *FuzzyScore, HTuple *IntraDistance, HTuple *InterDistance) const;
  // Extract straight edges perpendicular to a rectangle or an annular arc.
  virtual HTuple FuzzyMeasurePos(const HTuple &MeasureHandle, const HTuple &Sigma, const HTuple &AmpThresh, const HTuple &FuzzyThresh, const HTuple &Transition, HTuple *ColumnEdge, HTuple *Amplitude, HTuple *FuzzyScore, HTuple *Distance) const;
  virtual HTuple FuzzyMeasurePos(long MeasureHandle, double Sigma, double AmpThresh, double FuzzyThresh, const char *Transition, HTuple *ColumnEdge, HTuple *Amplitude, HTuple *FuzzyScore, HTuple *Distance) const;
  // Extract straight edge pairs perpendicular to a rectangle or annular arc.
  virtual HTuple MeasurePairs(const HTuple &MeasureHandle, const HTuple &Sigma, const HTuple &Threshold, const HTuple &Transition, const HTuple &Select, HTuple *ColumnEdgeFirst, HTuple *AmplitudeFirst, HTuple *RowEdgeSecond, HTuple *ColumnEdgeSecond, HTuple *AmplitudeSecond, HTuple *IntraDistance, HTuple *InterDistance) const;
  virtual HTuple MeasurePairs(long MeasureHandle, double Sigma, double Threshold, const char *Transition, const char *Select, HTuple *ColumnEdgeFirst, HTuple *AmplitudeFirst, HTuple *RowEdgeSecond, HTuple *ColumnEdgeSecond, HTuple *AmplitudeSecond, HTuple *IntraDistance, HTuple *InterDistance) const;
  // Extract straight edges perpendicular to a rectangle or annular arc.
  virtual HTuple MeasurePos(const HTuple &MeasureHandle, const HTuple &Sigma, const HTuple &Threshold, const HTuple &Transition, const HTuple &Select, HTuple *ColumnEdge, HTuple *Amplitude, HTuple *Distance) const;
  virtual HTuple MeasurePos(long MeasureHandle, double Sigma, double Threshold, const char *Transition, const char *Select, HTuple *ColumnEdge, HTuple *Amplitude, HTuple *Distance) const;
  // Image restoration by Wiener filtering.
  virtual HImage WienerFilterNi(const HImage &Psf, const HRegion &NoiseRegion, const HTuple &MaskWidth, const HTuple &MaskHeight) const;
  virtual HImage WienerFilterNi(const HImage &Psf, const HRegion &NoiseRegion, long MaskWidth, long MaskHeight) const;
  // Image restoration by Wiener filtering.
  virtual HImage WienerFilter(const HImage &Psf, const HImage &FilteredImage) const;
  // Generate an impulse response of a (linearly) motion blurring.
  static HImage GenPsfMotion(const HTuple &PSFwidth, const HTuple &PSFheight, const HTuple &Blurring, const HTuple &Angle, const HTuple &Type);
  static HImage GenPsfMotion(long PSFwidth, long PSFheight, double Blurring, long Angle, long Type);
  // Simulation of (linearly) motion blur.
  virtual HImage SimulateMotion(const HTuple &Blurring, const HTuple &Angle, const HTuple &Type) const;
  virtual HImage SimulateMotion(double Blurring, long Angle, long Type) const;
  // Generate an impulse response of an uniform out-of-focus blurring.
  static HImage GenPsfDefocus(const HTuple &PSFwidth, const HTuple &PSFheight, const HTuple &Blurring);
  static HImage GenPsfDefocus(long PSFwidth, long PSFheight, double Blurring);
  // Simulate an uniform out-of-focus blurring of an image.
  virtual HImage SimulateDefocus(const HTuple &Blurring) const;
  virtual HImage SimulateDefocus(double Blurring) const;
};

#endif
