/*****************************************************************************
 * HXLDCont.h
 ***************************************************************************** 
 *
 * Project:     Halcon/libhalcon
 * Description: Class HXLDCont
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.14 $
 * $Date: 2001/12/06 10:04:06 $
 *
 */

#ifndef XLD_CONT_H
#define XLD_CONT_H


/****************************************************************************/
/*                               HXLDCont                                   */
/****************************************************************************/

class HXLDContArray;
class LIntExport HXLDCont: public HXLD {
  friend class HXLDContArray;
public:
  HXLDCont(void): HXLD() {num_cont++;}
  HXLDCont(Hkey key);
  HXLDCont(const Hobject &obj);
  HXLDCont(const HXLDCont &cont);
  HXLDCont(const HXLDContArray &arr);
  HXLDCont(const HXLD &xld);
  ~HXLDCont(void) {num_cont--;}

  HXLDCont &operator= (const HXLDCont &cont);
  HXLDCont &operator= (const HXLDContArray &arr);
  HXLDCont &operator= (const HXLD &xld);

  // Tools
  virtual const char *HClassName(void) const;
  virtual const char *ClassName(void) const;
  virtual const char *InstClassName(void) const;
  virtual       INT4  ClassDBID(void) const;
  HBool         Shared(void) const;

  // Class Functions
  static long NumInst(void) {return num_cont;}

private:
  static long num_cont;

public:
  // Interactive modifikation of a contour.
  virtual HXLDCont DrawXldMod(const HTuple &WindowHandle, const HTuple &Rotate, const HTuple &Move, const HTuple &Scale, const HTuple &KeepRatio, const HTuple &Edit) const;
  virtual HXLDCont DrawXldMod(long WindowHandle, const char *Rotate, const char *Move, const char *Scale, const char *KeepRatio, const char *Edit) const;
  // Calculate the minimum distance between two contours.
  virtual double DistanceCcMin(const HXLDCont &Contour2, const HTuple &Mode) const;
  virtual double DistanceCcMin(const HXLDCont &Contour2, const char *Mode) const;
  virtual double DistanceCcMin(const HXLDContArray &Contour2, const HTuple &Mode) const;
  virtual double DistanceCcMin(const HXLDContArray &Contour2, const char *Mode) const;
  // Calculate the distance between two contours.
  virtual double DistanceCc(const HXLDCont &Contour2, const HTuple &Mode, double *DistanceMax) const;
  virtual double DistanceCc(const HXLDCont &Contour2, const char *Mode, double *DistanceMax) const;
  virtual double DistanceCc(const HXLDContArray &Contour2, const HTuple &Mode, double *DistanceMax) const;
  virtual double DistanceCc(const HXLDContArray &Contour2, const char *Mode, double *DistanceMax) const;
  // Calculate the distance between one line segment and one contour.
  virtual HTuple DistanceSc(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2, HTuple *DistanceMax) const;
  virtual HTuple DistanceSc(double Row1, double Column1, double Row2, double Column2, HTuple *DistanceMax) const;
  // Calculate the distance between one line and one contour.
  virtual HTuple DistanceLc(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2, HTuple *DistanceMax) const;
  virtual HTuple DistanceLc(double Row1, double Column1, double Row2, double Column2, HTuple *DistanceMax) const;
  // Calculate the distance between one point and one contour.
  virtual HTuple DistancePc(const HTuple &Row, const HTuple &Column, HTuple *DistanceMax) const;
  virtual HTuple DistancePc(double Row, double Column, HTuple *DistanceMax) const;
  // Change the radial distortion of contours.
  virtual HXLDCont ChangeRadialDistortionContoursXld(const HTuple &CamParIn, const HTuple &CamParOut) const;
  virtual HXLDCont ChangeRadialDistortionContoursXld(double CamParIn, double CamParOut) const;
  // Create a region from an XLD contour.
  virtual HRegion GenRegionContourXld(const HTuple &Mode) const;
  virtual HRegion GenRegionContourXld(const char *Mode) const;
  // Merge XLD contours from successive line scan images.
  virtual HXLDCont MergeContLineScanXld(const HXLDCont &PrevConts, HXLDCont *PrevMergedConts, const HTuple &ImageHeight, const HTuple &Margin, const HTuple &MergeBorder, const HTuple &MaxImagesCont) const;
  virtual HXLDCont MergeContLineScanXld(const HXLDCont &PrevConts, HXLDCont *PrevMergedConts, long ImageHeight, double Margin, const char *MergeBorder, long MaxImagesCont) const;
  virtual HXLDCont MergeContLineScanXld(const HXLDContArray &PrevConts, HXLDCont *PrevMergedConts, const HTuple &ImageHeight, const HTuple &Margin, const HTuple &MergeBorder, const HTuple &MaxImagesCont) const;
  virtual HXLDCont MergeContLineScanXld(const HXLDContArray &PrevConts, HXLDCont *PrevMergedConts, long ImageHeight, double Margin, const char *MergeBorder, long MaxImagesCont) const;
  // Read XLD contours to a file in ARC/INFO generate format.
  static HXLDCont ReadContourXldArcInfo(const HTuple &FileName);
  static HXLDCont ReadContourXldArcInfo(const char *FileName);
  // Write XLD contours to a file in ARC/INFO generate format.
  virtual void WriteContourXldArcInfo(const HTuple &FileName) const;
  virtual void WriteContourXldArcInfo(const char *FileName) const;
  // Segment XLD contours into line segments and circular or elliptic arcs.
  virtual HXLDContArray SegmentContoursXld(const HTuple &Mode, const HTuple &SmoothCont, const HTuple &MaxLineDist1, const HTuple &MaxLineDist2) const;
  virtual HXLDContArray SegmentContoursXld(const char *Mode, long SmoothCont, double MaxLineDist1, double MaxLineDist2) const;
  // Approximation of XLD contours by circles.
  virtual double FitCircleContourXld(const HTuple &Algorithm, const HTuple &MaxNumPoints, const HTuple &MaxClosureDist, const HTuple &ClippingEndPoints, const HTuple &Iterations, const HTuple &ClippingFactor, double *Column, double *Radius, double *StartPhi, double *EndPhi, char *PointOrder) const;
  virtual double FitCircleContourXld(const char *Algorithm, long MaxNumPoints, double MaxClosureDist, long ClippingEndPoints, long Iterations, double ClippingFactor, double *Column, double *Radius, double *StartPhi, double *EndPhi, char *PointOrder) const;
  // Approximation of XLD contours by line segments.
  virtual double FitLineContourXld(const HTuple &Algorithm, const HTuple &MaxNumPoints, const HTuple &ClippingEndPoints, const HTuple &Iterations, const HTuple &ClippingFactor, double *ColBegin, double *RowEnd, double *ColEnd, double *Nr, double *Nc, double *Dist) const;
  virtual double FitLineContourXld(const char *Algorithm, long MaxNumPoints, long ClippingEndPoints, long Iterations, double ClippingFactor, double *ColBegin, double *RowEnd, double *ColEnd, double *Nr, double *Nc, double *Dist) const;
  // Distance of contours to an ellipse.
  virtual double DistEllipseContourXld(const HTuple &Mode, const HTuple &MaxNumPoints, const HTuple &ClippingEndPoints, const HTuple &Row, const HTuple &Column, const HTuple &Phi, const HTuple &Radius1, const HTuple &Radius2, double *MaxDist, double *AvgDist, double *SigmaDist) const;
  virtual double DistEllipseContourXld(const char *Mode, long MaxNumPoints, long ClippingEndPoints, double Row, double Column, double Phi, double Radius1, double Radius2, double *MaxDist, double *AvgDist, double *SigmaDist) const;
  // Approximation of XLD contours by ellipses or elliptic arcs.
  virtual double FitEllipseContourXld(const HTuple &Algorithm, const HTuple &MaxNumPoints, const HTuple &MaxClosureDist, const HTuple &ClippingEndPoints, const HTuple &VossTabSize, const HTuple &Iterations, const HTuple &ClippingFactor, double *Column, double *Phi, double *Radius1, double *Radius2, double *StartPhi, double *EndPhi, char *PointOrder) const;
  virtual double FitEllipseContourXld(const char *Algorithm, long MaxNumPoints, double MaxClosureDist, long ClippingEndPoints, long VossTabSize, long Iterations, double ClippingFactor, double *Column, double *Phi, double *Radius1, double *Radius2, double *StartPhi, double *EndPhi, char *PointOrder) const;
  // Creation of an XLD contour corresponding to an elliptic arc.
  static HXLDCont GenEllipseContourXld(const HTuple &Row, const HTuple &Column, const HTuple &Phi, const HTuple &Radius1, const HTuple &Radius2, const HTuple &StartPhi, const HTuple &EndPhi, const HTuple &PointOrder, const HTuple &Resolution);
  static HXLDCont GenEllipseContourXld(double Row, double Column, double Phi, double Radius1, double Radius2, double StartPhi, double EndPhi, const char *PointOrder, double Resolution);
  // Add noise to XLD contours.
  virtual HXLDCont AddNoiseWhiteContourXld(const HTuple &NumRegrPoints, const HTuple &Amp) const;
  virtual HXLDCont AddNoiseWhiteContourXld(long NumRegrPoints, double Amp) const;
  // Apply a projective transformation to an XLD contour.
  virtual HXLDCont ProjectiveTransContourXld(const HTuple &HomMat2D) const;
  virtual HXLDCont ProjectiveTransContourXld(double HomMat2D) const;
  // Apply an arbitrary affine 2D transformation to XLD contours.
  virtual HXLDCont AffineTransContourXld(const HTuple &HomMat2D) const;
  virtual HXLDCont AffineTransContourXld(double HomMat2D) const;
  // Calculate the direction of an XLD contour for each contour point.
  virtual HTuple GetContourAngleXld(const HTuple &AngleMode, const HTuple &CalcMode, const HTuple &Lookaround) const;
  virtual HTuple GetContourAngleXld(const char *AngleMode, const char *CalcMode, long Lookaround) const;
  // Return the number of points in an XLD contour.
  virtual long ContourPointNumXld(void) const;
  // Return the names of the defined global attributes of an XLD contour.
  virtual HTuple QueryContourGlobalAttribsXld(void) const;
  // Return global attributes values of an XLD contour.
  virtual HTuple GetContourGlobalAttribXld(const HTuple &Name) const;
  virtual HTuple GetContourGlobalAttribXld(const char *Name) const;
  // Return the names of the defined attributes of an XLD contour.
  virtual HTuple QueryContourAttribsXld(void) const;
  // Return point attribute values of an XLD contour.
  virtual HTuple GetContourAttribXld(const HTuple &Name) const;
  virtual HTuple GetContourAttribXld(const char *Name) const;
  // Return the coordinates of an XLD contour.
  virtual HTuple GetContourXld(HTuple *Col) const;
  // Generate a XLD contour with rounded corners from a polygon (given
  // as tuples).
  static HXLDCont GenContourPolygonRoundedXld(const HTuple &Row, const HTuple &Col, const HTuple &Radius, const HTuple &SamplingInterval);
  static HXLDCont GenContourPolygonRoundedXld(double Row, double Col, double Radius, double SamplingInterval);
  // Generate an XLD contour from a polygon (given as tuples).
  static HXLDCont GenContourPolygonXld(const HTuple &Row, const HTuple &Col);
  static HXLDCont GenContourPolygonXld(double Row, double Col);
};

#endif
